/****************************************************************************

  Testprogram for SPI-EEPROM-Driver
  PLCmodule C14

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29

  08.12.2000   -ct

****************************************************************************/


#include "generic/global.h"
#include "../include/c14drv.h"
#include "../include/eeprmdrv.h"
#include "../include/timerdrv.h"
#include <stdio.h>


#ifdef  __C51__
   #include <reg505c.h>

#elif  __C166__
   #include <xc161.h>
#endif



//---------------------------------------------------------------------------
//  Definition
//---------------------------------------------------------------------------

#define EEPROM_TYPE     EPRM25256
#define BUFFER_SIZE     0x10000
#define SIZE_CFG_DATA   0x80
#define SIZE_PROC_DATA  0x100

#define EA              PSW_IEN
#define RxINT           ASC0_RIC_IR
#define RxBUF           ASC0_RBUF




//---------------------------------------------------------------------------
//  Deklaration internal prototypes
//---------------------------------------------------------------------------

void  PrintData (BYTE FAR* pDataBuffer_p, WORD wPrintSize_p);
char  ReadChar (void);


BYTE  FAR WriteBuffer_g[BUFFER_SIZE];
BYTE  FAR ReadBuffer_g[BUFFER_SIZE];


/****************************************************************************
*                                                                           *
*                   H A U P T P R O G R A M                                 *
*                                                                           *
****************************************************************************/

void main (void)
{

tEprmSectSize  EprmSectSize;
tEprmInitCfg   EprmInitCfg;
BYTE   bFillByte;
BYTE   bRes;
BYTE   bEepromType;
WORD   wEprmDrvVer;
DWORD  dwTime;
DWORD  dwEprmProgSize;
DWORD  dwEprmProcSize;
DWORD  dwIdx;


    // check for initialized ASC0
    if (ASC0_CON == 0)
    {
        //  Configuration of the ASC0 Baudrate Generator:
        //  - fractional divider as prescaler for baud rate timer is used
        ASC0_FDV       =  0x01D0;     // load ASC0 fractional divider register
                                      //  - required baud rate = 38,400 kbaud
                                      //  - real baud rate     = 38,400 kbaud
                                      //  - deviation          = 0,001 %
    
        ASC0_BG        =  0x003A;     // load ASC0 baud rate time reload register
    
    
        //  Configuration of the ASC0 Operation Mode:
        //  - 8-bit data asychronous operation with one stop bit
        //  - receiver is enabled
        //  - fractional divider is enabled
        ASC0_CON       =  0x0811;     // load ASC0 control register
    
        //  FIFO Configuration:
        //  - receive FIFO is disabled
        //  - transmit FIFO is disabled
        ASC0_RXFCON    =  0x0102;     // load ASC0 receive FIFO control register
        ASC0_TXFCON    =  0x0102;     // load ASC0 transmit FIFO control register
    
        //  Configuration of the used ASC0 Port Pins:
        //  - P3.10 is used for ASC Transmit Data Output (TxDA0)
        //  - P3.11 is used for ASC0 Receive data Input (RxDA0)
        ALTSEL0P3     |=  0x0400;      // select alternate output function
        P3   = (P3   & ~(WORD)0x0400) | 0x0400;    //set data register
        DP3  = (DP3  & ~(WORD)0x0400) | 0x0400;    //set direction register
    
        //  Configuration of the used ASC0 Interrupts:
        ASC0_TIC |= 0x00080;
    
        ASC0_CON      |=  0x8000;      // enable baud rate generator
    }

    C14Initialize();

    //-----------------------------------------------------------------------
    // Initialize and get the size
    //-----------------------------------------------------------------------
    printf("\n\nTestprogram for SPI-EEPROM-driver\n");

    // general Init's
    printf("\init Timer ... ");
    StartTimer();
    printf("OK");

    // enable interrupts
    PSW_IEN = 1;


    bEepromType = EEPROM_TYPE;
//    bEepromType |= TWO_EEPROM;

    EprmInitCfg.m_bDeviceType    = bEepromType;
    EprmInitCfg.m_dwCfgSectSize  = SIZE_CFG_DATA;
    EprmInitCfg.m_dwPdatSectSize = SIZE_PROC_DATA;


    // driver for SPI-EEPROM initialize
    printf ("\n\n\nInit the SPI-EEPROM-driver ...");
    wEprmDrvVer = EprmInit (&EprmInitCfg);
    printf ("\ndriver-version: %1X.%02X",
           (wEprmDrvVer / 0x100), (wEprmDrvVer & 0x00FF));


    // get usable data-areas
    EprmGetSectSize(&EprmSectSize);
    printf ("\ndata-area for config-data  [Byte]: 0x%04lX/%ludez",
            EprmSectSize.m_dwCfgSectSize, EprmSectSize.m_dwCfgSectSize);
    printf ("\ndata-area for program [Byte]:      0x%04lX/%ludez",
            EprmSectSize.m_dwProgSectSize, EprmSectSize.m_dwProgSectSize);
    printf ("\ndata-area for process-data [Byte]: 0x%04lX/%ludez",
            EprmSectSize.m_dwPdatSectSize, EprmSectSize.m_dwPdatSectSize);

    dwEprmProgSize = EprmSectSize.m_dwProgSectSize;
    dwEprmProcSize = EprmSectSize.m_dwPdatSectSize;


    if (dwEprmProgSize > BUFFER_SIZE)
    {
        printf ("\n\n\nERROR: <BufferSize> to small !");
        while(1);
    }


    //-----------------------------------------------------------------------
    // check validation of datas in eeprom
    //-----------------------------------------------------------------------
    if ( EprmCheckDataValid() == TRUE )
    {
        printf ("\nData in EEPROM valid");
        printf ("\nread data...");
        bRes = EprmRecoverData (&ReadBuffer_g[0], dwEprmProcSize, dwEprmProgSize);
//        PrintData (&ReadBuffer_g[0], (WORD)dwEprmProgSize);
    }

    else
    {
        printf ("\nEEPROM data invalid");
    }


    printf("\ndelete data... ");
    bRes = EprmSetDataInvalid();
    if (bRes == TRUE)
    {
        printf("\nOK");
    }


    if ( EprmCheckDataValid() == TRUE )
    {
        printf ("\ndata in EEPROM valid");
    }

    else
    {
        printf ("\ndata in EEPROM invalid");
    }


    //-----------------------------------------------------------------------
    // save data in EEPROM
    //-----------------------------------------------------------------------
    printf ("\npress a key on the pc-keyboard (=start value for memory data): ");
    bFillByte = ReadChar();
    printf ("\nstart value = '%c'/%02X", bFillByte, (WORD)bFillByte);

    for (dwIdx=0; dwIdx<BUFFER_SIZE; dwIdx++)
    {
        WriteBuffer_g[dwIdx] = bFillByte++;
        ReadBuffer_g[dwIdx]  = 0xFF;
    }

//    PrintData (&WriteBuffer_g[0], (WORD)dwEprmProgSize);

    //  Daten im EEPROM sichern
    printf("\nsave data in EEPROM... ");
    dwTime = GetTickCount();
    bRes   = EprmSaveData (&WriteBuffer_g[0], dwEprmProcSize, dwEprmProgSize);
    if ( bRes )
    {
        bRes = EprmSetDataValid();
    }
    dwTime = GetTickCount() - dwTime;

    if ( bRes )
    {
        printf("\nOK");
    }
    printf("\nprocessing time = %lums", dwTime);


    //-----------------------------------------------------------------------
    // Daten aus EEPROM zurcklesen
    //-----------------------------------------------------------------------
    printf("\nread data from EEPROM... ");
    dwTime = GetTickCount();
    bRes   = EprmRecoverData (&ReadBuffer_g[0], dwEprmProcSize, dwEprmProgSize);
    dwTime = GetTickCount() - dwTime;

    if (bRes == TRUE)
    {
        printf("\nOK");
    }
    printf("\nprocessing time = %lums", dwTime);

//   PrintData (&ReadBuffer_g[0], (WORD)dwEprmProgSize);


    //-----------------------------------------------------------------------
    // in EEPROM geschriebene und zurckgelese Daten vergleichen
    //-----------------------------------------------------------------------
    printf("\ncompare data:");
    bRes = TRUE;
    for (dwIdx=0; dwIdx<dwEprmProgSize; dwIdx++)
    {
        if (ReadBuffer_g[dwIdx] != WriteBuffer_g[dwIdx])
        {
            printf("\ndata on offset 0x%04lX different: ", dwIdx);
            printf("Wr=%02X/Rd=%02X",
                  (WORD)WriteBuffer_g[dwIdx],(WORD)ReadBuffer_g[dwIdx]);
            bRes = FALSE;
        }
    }

    if ( bRes )
    {
        printf("\nwritten and read datas are identical");
    }


    //-----------------------------------------------------------------------
    // end of program
    //-----------------------------------------------------------------------
    printf("\n\nend of program\n");
    while(1);

}





/****************************************************************************
*                                                                           *
*                   Internal auxiliary functions                            *
*                                                                           *
****************************************************************************/

//---------------------------------------------------------------------------
//  output the readed data
//---------------------------------------------------------------------------

void  PrintData (BYTE FAR* pDataBuffer_p, WORD wPrintSize_p)
{

WORD  i, j;


    printf ("\ndata:");

    for (i=0; wPrintSize_p; i++)
    {
        printf ("\n%04X:   ", i*0x10);

        for (j=0; j<16; j++)
        {
            printf ("%02X ", (WORD)*pDataBuffer_p++);

            if ( !--wPrintSize_p )
            {
                break;
            }
        }
    }

}



//---------------------------------------------------------------------------
//  input char
//---------------------------------------------------------------------------


char  ReadChar (void)
{

#define BCKSPCE     0x08
#define ENTER       0x0D
#define FIRST_ASCII 0x20
#define LAST_ASCII  0x9B


BYTE  c;


    do
    {
        while (!RxINT);

        c = (BYTE)RxBUF;
        RxINT = 0;
    }
    while ((c!=ENTER) && (c!=BCKSPCE) && ((c<FIRST_ASCII) || (c>LAST_ASCII)));

    return (c);

}





// EOF

