/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  Implementation of EEPROM driver API functions

  -------------------------------------------------------------------------

  1999/01/08 -rs:   start of implemtation
  2001/12/10 -ct:   adjust for the DIPmodul 164
  2003/10/28 -ct:   adjust for the XC161
  2003/06/07 -rs:   V2.10 (additional sector for PLC configuration included)

****************************************************************************/

#include "generic/global.h"
#include "../include/eeprmdrv.h"
#include "../include/eepromio.h"
#include <string.h>





/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define DATA_VALID_SIGNUM   "SYS#PLC"   // identification for "data valid"



//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static tEprm  Eprom_g;         // descriptor for all relevant eeprom parameter



//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------





/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <EEPROM Driver>                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define OFFS_PLCCFG     0



//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static  DWORD  dwCfgSectOffs_l;
static  DWORD  dwCfgSectSize_l;
static  DWORD  dwValSignOffs_l;
static  DWORD  dwValSignSize_l;
static  DWORD  dwProgSectOffs_l;
static  DWORD  dwProgSectSize_l;
static  DWORD  dwPdatSectOffs_l;
static  DWORD  dwPdatSectSize_l;

static  BYTE   szSignumBuff_l[sizeof(DATA_VALID_SIGNUM)];



//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------





//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     EprmInit
//
// Description:  This function initialize the eeprom driver
//
// Parameters:   pEprmInitCfg_p: pointer to configuration strucgture
// 
// Returns:      driver version
//
// State:        o.k. 2003/06/07 -rs
//
//---------------------------------------------------------------------------
// Note: In the higest bit of "pEprmInitCfg_p->bDeviceType_p" is the count
//       of mounted eeproms encoded. A "1" indicates that 2 eeproms are
//       mounted and a "0" indicates that 1 eeprom is present
//---------------------------------------------------------------------------

WORD  EprmInit (tEprmInitCfg FAR* pEprmInitCfg_p)
{

BYTE   bDeviceType;
DWORD  dwEepromSize;
DWORD  dwProgSectSize;


    bDeviceType = pEprmInitCfg_p->m_bDeviceType;

    Eprom_g.m_bType     = (BYTE)(bDeviceType & ~TWO_EEPROM);
    Eprom_g.m_bAmount   = 0;
    Eprom_g.m_bPageSize = EEPGetPageSize(Eprom_g.m_bType);
    Eprom_g.m_dwEndAdr  = EEPGetEndAdr(Eprom_g.m_bType);
    Eprom_g.m_bBaudRate = EEPGetBaudrate(Eprom_g.m_bType);
    dwEepromSize        = EEPGetSize (Eprom_g.m_bType);

    if ( (bDeviceType & TWO_EEPROM) )
    {
        Eprom_g.m_bAmount = 1;
        dwEepromSize *= 2;
    }


    // initialize low-level driver
    EepromInit (Eprom_g.m_bBaudRate);


    dwProgSectSize  = dwEepromSize;
    dwProgSectSize -= OFFS_PLCCFG + pEprmInitCfg_p->m_dwCfgSectSize;
    dwProgSectSize -= sizeof(DATA_VALID_SIGNUM);
    dwProgSectSize -= pEprmInitCfg_p->m_dwPdatSectSize;

    dwCfgSectOffs_l  = OFFS_PLCCFG;
    dwCfgSectSize_l  = pEprmInitCfg_p->m_dwCfgSectSize;
    dwValSignOffs_l  = dwCfgSectOffs_l + dwCfgSectSize_l;
    dwValSignSize_l  = sizeof(DATA_VALID_SIGNUM);
    dwProgSectOffs_l = dwValSignOffs_l + dwValSignSize_l;
    dwProgSectSize_l = dwProgSectSize;
    dwPdatSectOffs_l = dwProgSectOffs_l + dwProgSectSize_l;
    dwPdatSectSize_l = pEprmInitCfg_p->m_dwPdatSectSize;


    return ( (DOMAIN << 8) | REVISON );

}



//---------------------------------------------------------------------------
//
// Function:     EprmGetSectSize
//
// Description:  This function returns the size of various logical sectors
//               of the EEPROM
//
// Parameters:   pEprmSectSize_p: pointer to structure with members for
//                                for all sectors
// 
// Returns:      TRUE if ok, FALSE otherwise
//
// State:        o.k. 2004/06/07 -rs
//
//---------------------------------------------------------------------------

BYTE  EprmGetSectSize (tEprmSectSize FAR* pEprmSectSize_p)
{

    pEprmSectSize_p->m_dwCfgSectSize  = dwCfgSectSize_l;
    pEprmSectSize_p->m_dwProgSectSize = dwProgSectSize_l;
    pEprmSectSize_p->m_dwPdatSectSize = dwPdatSectSize_l;

    return (TRUE);

}



//---------------------------------------------------------------------------
//
// Function:     EprmSavePlcCfg
//
// Description:  This function saves the PLC configuration to eeprom
//               Notice: The PLC configuration itself depends on the
//                       device type (Z3/Z4/Z5, ...) and is defined in
//                       the configuration of the runtime environment
//                       of the PLC. The function <EprmInit> reserves
//                       the necessary EEPROM sector.
//
// Parameters:   pSrcData_p:     pointer to structure with configuration data
//                               which should be saved
//               dwDataOffset_p: start addres in the eeprom
//               dwDataSize_p:   number of bytes to save
// 
// Returns:      TRUE if ok, FALSE otherwise
//
// State:        o.k. 2004/06/07 -rs
//
//---------------------------------------------------------------------------

#ifdef _SUPPORT_PLC_CFG_SECT_

BYTE  EprmSavePlcCfg (BYTE FAR* pSrcData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p)
{

BYTE  bRes;


    if ((dwDataOffset_p + dwDataSize_p) > dwCfgSectSize_l)
    {
        return (FALSE);
    }


    bRes = EepromWrBlk (pSrcData_p, dwCfgSectOffs_l+dwDataOffset_p,
                        dwDataSize_p, (tEprm FAR*)&Eprom_g);
    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}



//---------------------------------------------------------------------------
//
// Function:     EprmSavePlcCfg
//
// Description:  This function recovers the PLC configuration from eeprom
//               Notice: The PLC configuration itself depends on the
//                       device type (Z3/Z4/Z5, ...) and is defined in
//                       the configuration of the runtime environment
//                       of the PLC. The function <EprmInit> reserves
//                       the necessary EEPROM sector.
//
// Parameters:   pDstData_p:     pointer to structure with configuration data
//                               which should be recoverd
//               dwDataOffset_p: start addres in the eeprom
//               dwDataSize_p:   number of bytes to recover
// 
// Returns:      TRUE if ok, FALSE otherwise
//
// State:        o.k. 2004/06/07 -rs
//
//---------------------------------------------------------------------------

BYTE  EprmRecoverPlcCfg (BYTE FAR* pDstData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p)
{

BYTE  bRes;


    if ((dwDataOffset_p + dwDataSize_p) > dwCfgSectSize_l)
    {
        return (FALSE);
    }


    bRes = EepromRdBlk (pDstData_p, dwCfgSectOffs_l+dwDataOffset_p,
                        dwDataSize_p, (tEprm FAR*)&Eprom_g);
    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}

#endif  // #ifdef _SUPPORT_PLC_CFG_SECT_



//---------------------------------------------------------------------------
//
// Function:     EprmSetDataValid
//
// Description:  This function set data in the eeprom valid
//
//
// Parameters:   none
//
// 
// Returns:      status
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------

BYTE  EprmSetDataValid (void)
{

BYTE  bRes;


    // set data in eeprom valid
    bRes = EepromWrBlk (DATA_VALID_SIGNUM, dwValSignOffs_l,
                        sizeof(DATA_VALID_SIGNUM), &Eprom_g);

    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}



//---------------------------------------------------------------------------
//
// Function:     EprmSetDataInvalid
//
// Description:  This function set data in the eeprom invalid
//
//
// Parameters:   none
//
// 
// Returns:      state 
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------

BYTE  EprmSetDataInvalid (void)
{

BYTE  bRes;
BYTE  nIdx;


    // clear the valid field
    for (nIdx=0; nIdx<sizeof(szSignumBuff_l); nIdx++)
    {
        szSignumBuff_l[nIdx] = 0;
    }

    // write the valid field into the eeprom
    bRes = EepromWrBlk (&szSignumBuff_l[0], dwValSignOffs_l,
                        sizeof(szSignumBuff_l), &Eprom_g);
    
    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}



//---------------------------------------------------------------------------
//
// Function:     EprmCheckDataValid
//
// Description:  This function look and check for valid data in the eeprom
//
//
// Parameters:   none
//
// 
// Returns:      state
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------

BYTE  EprmCheckDataValid (void)
{

BYTE  bRes;


    // read the valid field out of the eeprom
    bRes = EepromRdBlk (&szSignumBuff_l[0], dwValSignOffs_l,
                        sizeof(szSignumBuff_l), &Eprom_g);
    if (bRes != 0)
    {
        // error at eeprom access
        return (FALSE);
    }


    // check the valid field for validity
    bRes = (BYTE)memcmp (&szSignumBuff_l[0], DATA_VALID_SIGNUM,
                         sizeof(DATA_VALID_SIGNUM));

    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}



//---------------------------------------------------------------------------
//
// Function:     EprmSaveData
//
// Description:  This function save data into the eeprom
//
//
// Parameters:   pSrcData_p     : pointer to the source data
//               dwDataOffset_p : start addres in the eeprom
//               dwDataSize_p   : count of bytes
//
// Returns:      state
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------

BYTE  EprmSaveData (BYTE FAR* pSrcData_p, DWORD dwDataOffset_p,
                    DWORD dwDataSize_p)
{

BYTE  bRes;


    if ((dwDataOffset_p + dwDataSize_p) > (dwProgSectSize_l + dwPdatSectSize_l))
    {
        return (FALSE);
    }


    bRes = EepromWrBlk (pSrcData_p, dwProgSectOffs_l+dwDataOffset_p,
                        dwDataSize_p, &Eprom_g);
    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }

}



//---------------------------------------------------------------------------
//
// Function:     EprmRecoverData
//
// Description:  This function read data out of the eeprom
//
//
// Parameters:   pDstData_p     : destination pointer for the data
//               dwDataOffset_p : start addres in the eeprom
//               dwDataSize_p   : count of bytes
//
// Returns:      state
//
//
// State:        o.k. 2003/10/28 ct 
//
//---------------------------------------------------------------------------

BYTE  EprmRecoverData (BYTE FAR* pDstData_p, DWORD dwDataOffset_p,
                       DWORD dwDataSize_p)
{

BYTE  bRes;


    if ((dwDataOffset_p + dwDataSize_p) > (dwProgSectSize_l + dwPdatSectSize_l))
    {
        return (FALSE);
    }


    bRes = EepromRdBlk (pDstData_p, dwProgSectOffs_l+dwDataOffset_p,
                        dwDataSize_p, &Eprom_g);
    if (bRes == 0)
    {
        return (TRUE);
    }
    else
    {
        return (FALSE);
    }
    
}




// EOF


