/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  source file for memory driver

  -------------------------------------------------------------------------

                $RCSfile: MdvSeDrv.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1 $  $Date: 2004/10/06 11:25:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/09/24 -as:   start of the implemtation

****************************************************************************/

//---------------------------------------------------------------------------
// global includes
#include "generic/global.h"
#include "generic/gmacros.h"

//---------------------------------------------------------------------------
// modulspecific includes
#include "../include/C14drv.h"
#include "../include/MdvSeDrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static WORD SpiInit     (WORD wBaudrate_p);
static void SpiReadPage (BYTE FAR* pbDstPtr_p, DWORD dwSrcAdr_p, DWORD dwSize_p, BYTE bEpromNr_p);
static void SpiWritePage(BYTE FAR* pbSrcPtr_p, DWORD dwDstAdr_p, DWORD dwSize_p, BYTE bEpromNr_p);



/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <memory driver>                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static tMdvSeCfg MdvSeCfg_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static DWORD MdvSeGetEndAdr   (BYTE bDeviceType_p);
static BYTE  MdvSeGetPageSize (BYTE bDeviceType_p);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     MdvSeInit
//
// Description:  This function initialize the driver
//
//
// Parameters:   memory device configuration
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC MdvSeInit (tMdvSeCfg *pDevCfg_p)
{
    // If selected device supported by driver?
    switch (pDevCfg_p->m_bType)
    {
        case EPRM25080 :
        case EPRM25160 :
        case EPRM25170 :
        case EPRM25640 :
        case EPRM25650 :
        case EPRM25128 :
        case EPRM25256 : break;

        case EPRM25512 :
        default        : return (MDV_UNVALID_DEVICE_TYPE);
    }

    // default value selected ?
    if (pDevCfg_p->m_bBaudRate == 0)
       pDevCfg_p->m_bBaudRate = BR2M;

    // get the last access address (size) of device
    pDevCfg_p->m_dwEndAdr = MdvSeGetEndAdr (pDevCfg_p->m_bType);
    if (!pDevCfg_p->m_dwEndAdr)
       return (MDV_UNVALID_DEVICE_TYPE);

    // get the page size of device
    pDevCfg_p->m_bPageSize = MdvSeGetPageSize (pDevCfg_p->m_bType);
    if (!pDevCfg_p->m_bPageSize)
       return (MDV_UNVALID_DEVICE_TYPE);

    // Anfrage, ob Schnittstelle bereits init?
    SpiInit((WORD)pDevCfg_p->m_bBaudRate);

    MdvSeCfg_l = *pDevCfg_p;

    return (MDV_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     MdvSeWriteData
//
// Description:  This function writes a number of bytes into the memory device.
//
//
// Parameters:   pbSrcPtr_p : pointer to the source data
//               dwDstAdr_p : destination address in the eeprom
//               dwSize_p   : number of bytes to write
//
// Returns:      returns the number of written bytes
//               -1: unvalid access address
//                0: no data written
//               >0: successful written the number of bytes
//
// State:        
//
//---------------------------------------------------------------------------

DWORD PUBLIC MdvSeWriteData (BYTE GENERIC* pbSrcPtr_p, DWORD dwDstAdr_p, DWORD dwSize_p)
{
DWORD dwWriteDataSize;
DWORD dwPageOffset;
DWORD dwWrittenBytes = 0;

    if((dwDstAdr_p + dwSize_p -1) > MdvSeCfg_l.m_dwEndAdr)
    {
      // error: address overflow
      return (-1L);
    }

    do
    {
        // get the next page border
        dwPageOffset = ((dwDstAdr_p + MdvSeCfg_l.m_bPageSize) & ~(MdvSeCfg_l.m_bPageSize - 1));
        
        // number of bytes to the next page border
        dwWriteDataSize = dwPageOffset - dwDstAdr_p;
        
        // If write less data than the dwWriteDataSize are, then write all the data. 
        // Otherwise write data to the next border and then always page size.
        if(dwWriteDataSize > dwSize_p)
        {
            dwWriteDataSize = dwSize_p;
        }
        
        SpiWritePage(pbSrcPtr_p, dwDstAdr_p, dwWriteDataSize, MdvSeCfg_l.m_bDeviceID);
        
        dwSize_p       -= dwWriteDataSize;
        dwDstAdr_p     += dwWriteDataSize;
        pbSrcPtr_p     += dwWriteDataSize;
        dwWrittenBytes += dwWriteDataSize;
    
    } while(dwSize_p != 0);
    
    return(dwWrittenBytes);
}

//---------------------------------------------------------------------------
//
// Function:     MdvSeReadData
//
// Description:  This function reads a number of bytes out of the eeprom
//
// Parameters:   pbDstPtr_p : pointer to the destination area in RAM
//               dwSrcAdr_p : source address in the eeprom
//               dwSize_p   : number of data bytes
// 
// Returns:      returns the number of read bytes
//               -1: unvalid access address
//                0: no data read
//               >0: successful read the number of bytes
//
//
// State:        
//
//---------------------------------------------------------------------------
DWORD PUBLIC MdvSeReadData (BYTE GENERIC* pbDstPtr_p, DWORD dwSrcAdr_p, DWORD dwSize_p)
{
    if(dwSrcAdr_p > MdvSeCfg_l.m_dwEndAdr)
    {
       // error: unvalid access address
       return (-1L);
    }
    
    if((dwSrcAdr_p + dwSize_p - 1) > MdvSeCfg_l.m_dwEndAdr)
    {
       // read data until the end of the eeprom
       dwSize_p = MdvSeCfg_l.m_dwEndAdr - dwSrcAdr_p + 1;
    }
    
    // read data
    SpiReadPage(pbDstPtr_p, dwSrcAdr_p, dwSize_p, MdvSeCfg_l.m_bDeviceID);
    
    return(dwSize_p);
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     MdvSeGetSize
//
// Description:  This function delivers the memory size of the device.
//
// Parameters:   bDeviceType_p : device type
//
// Returns:      memory size
//
//
// State:        
//
//---------------------------------------------------------------------------
static DWORD MdvSeGetSize (BYTE bDeviceType_p)
{
    switch (bDeviceType_p)
    {
        case EPRM25080 :  return (1024);            //  1 kByte
        case EPRM25160 :  return (2048);            //  2 kByte
        case EPRM25170 :  return (2048);            //  2 kByte
        case EPRM25640 :  return (8192);            //  8 kByte
        case EPRM25650 :  return (8192);            //  8 kByte
        case EPRM25128 :  return (16384);           // 16 kByte
        case EPRM25256 :  return (32768);           // 32 kByte
        case EPRM25512 :  return (0);               // 64 kByte
        default:          return (0);
    }
}

//---------------------------------------------------------------------------
//
// Function:     MdvSeGetPageSize
//
// Description:  This function delivers the page size of the device.
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      page size
//
//
// State:        
//
//---------------------------------------------------------------------------
static BYTE MdvSeGetPageSize (BYTE bDeviceType_p)
{
    switch (bDeviceType_p)
    {
        case EPRM25080 :  return (16);           //  16 Byte Page
        case EPRM25160 :  return (16);           //  16 Byte Page
        case EPRM25170 :  return (32);           //  32 Byte Page
        case EPRM25640 :  return (32);           //  32 Byte Page
        case EPRM25650 :  return (32);           //  32 Byte Page
        case EPRM25128 :  return (64);           //  64 Byte Page
        case EPRM25256 :  return (64);           //  64 Byte Page
        case EPRM25512 :  return (0);            // 128Byte Page
        default:          return (0);
    }

}

//---------------------------------------------------------------------------
//
// Function:     MdvSeGetEndAdr
//
// Description:  This function delivers the max. memory address of the 
//               device.
//
// Parameters:   bDeviceType_p : eeprom typ
//
// Returns:      max. memory address
//
//
// State:        
//
//---------------------------------------------------------------------------
static DWORD MdvSeGetEndAdr (BYTE bDeviceType_p)
{
    return (MdvSeGetSize(bDeviceType_p) - 1);
}


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <driver for SSC0>                                   */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Functions for the access to the internal SSC0 iinterface of the XC161
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define DOMAIN_SPI      0x01
#define REVISION_SPI    0x00

// SPI eeprom command 
#define CMD_WRSR  0x1             // Write status register
#define CMD_WRITE 0x2             // Write data to memeory array beginning at 
                                  // selected adress
#define CMD_READ  0x3             // Read data from memeory array beginning at
                                  // selected adress
#define CMD_WRDI  0x4             // Reset the write enable latch 
                                  // (disable write operations)
#define CMD_RDSR  0x5             // Read status register
#define CMD_WREN  0x6             // Set the write enable latch
                                  // (enable write operations)
#define CMD_DUMMY 0xFF            // DUMMY byte
#define WIPBIT    0x0             // Statuswert vom WIP-Bit

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

#ifndef _tWORD

#define _tWORD

    typedef union
    {
      WORD w;
      struct
        {
        BYTE low;
        BYTE high;
        } b;
    
    }tWORD;

#endif    

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static BYTE huge abCommand_l[128+3];

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static void SpiWren   (BYTE bEPROMNR_p);
static void SpiWrdi   (BYTE bEPROMNR_p);
static BYTE SpiRdsr   (BYTE bEPROMNR_p);
static void SpiWipPoll(BYTE bEPROMNR_p);


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SpiInit
//
// Description:  This fuction initializes the internal SSC0 interface
//
//
// Parameters:   baudrate for the SSC0 communication
//
// 
// Returns:      version of the driver
//
//
// State:        o.k. 2003/10/28 ct
//
//---------------------------------------------------------------------------
static WORD SpiInit(WORD wBaudrate_p)
{
WORD wTmp;

    wTmp = DOMAIN_SPI;
    wTmp <<= 8;
    wTmp |= REVISION_SPI;
    
    return(wTmp); 
}

//---------------------------------------------------------------------------
//
// Function:     SpiReadPage
//
// Description:  This fuction reads data out of the spi device
//
//
// Parameters:   pbDstPtr_p : pointer for the destination buffer in the RAM
//               dwSrcAdr_p : source address in the spi device
//               dwSize_p   : count of data to read
//               bEpromNr_p : number of the spi device
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiReadPage (BYTE FAR* pbDstPtr_p, DWORD dwSrcAdr_p, DWORD dwSize_p, BYTE bEpromNr_p)
{
DWORD dwIndex;
BYTE  abCommand[4];

/*
    TRACE1("\nSpiReadPage -> pbDstPtr_p : 0x%08X",(BYTE FAR*)pbDstPtr_p);
    TRACE1("\nSpiReadPage -> dwSrcAdr_p : 0x%08X",dwSrcAdr_p);
    TRACE1("\nSpiReadPage -> dwSize_p   : 0x%08X",dwSize_p);
    TRACE1("\nSpiReadPage -> bEpromNr_p : 0x%08X",bEpromNr_p);
*/
    //------------------------------------------------------------------
    // Ressource parametrieren: Sendepuffer, Empfangspuffer deklarieren
    // Mu nur einmal ausgefhrt werden, da sich die Parameter nicht ndern.
    C14SpiInitRessource (bEpromNr_p, 4, abCommand, abCommand, NULL);
    
    for (dwIndex = 0; dwIndex < dwSize_p; dwIndex++)
    {        
        abCommand[0] = CMD_READ;  
        abCommand[1] = (BYTE)(dwSrcAdr_p>>8);  
        abCommand[2] = (BYTE)dwSrcAdr_p;
        abCommand[3] = CMD_DUMMY;

        C14SpiStartSingleCycle(bEpromNr_p);

        while (C14SpiGetRrcState(bEpromNr_p) == SPI_DEVICE_BUSY);

        // TRACE2("\nSpiReadPage -> Daten 0x%02X auf Adresse 0x%04X", abCommand[3], (WORD)dwSrcAdr_p);   
        
        pbDstPtr_p[dwIndex] = abCommand[3];                 // read the next byte
        
        dwSrcAdr_p++;                                       // increment the source adress
    }
}

//---------------------------------------------------------------------------
//
// Function:     SpiWritePage
//
// Description:  This fuction writes data into the spi device. The size of
//               data can max. increase to the size of one eeprom page size.
//
// Parameters:   pbSrcPtr_p : pointer to the source data in RAM
//               dwDstAdr_p : destination address in the spi device
//               dwSize_p   : count of data to write
//               bEpromNr_p : number of the spi device
//
// Returns:      version of the driver
//
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWritePage(BYTE FAR* pbSrcPtr_p, DWORD dwDstAdr_p, DWORD dwSize_p, BYTE bEpromNr_p)
{
WORD  wIndex;

/* 
    TRACE1("\nSpiWritePage -> pbSrcPtr_p : 0x%08X",(BYTE FAR*)pbSrcPtr_p);
    TRACE1("\nSpiWritePage -> dwDstAdr_p : 0x%08X",dwDstAdr_p);
    TRACE1("\nSpiWritePage -> dwSize_p   : 0x%08X",dwSize_p);
    TRACE1("\nSpiWritePage -> bEpromNr_p : 0x%08X",bEpromNr_p);
*/
    //------------------------------------------------------------------
    // set the write enable bit
    SpiWren(bEpromNr_p);

    //------------------------------------------------------------------
    abCommand_l[0] = CMD_WRITE;  
    abCommand_l[1] = (BYTE)(dwDstAdr_p>>8);  
    abCommand_l[2] = (BYTE)dwDstAdr_p;

    for (wIndex = 0; wIndex < dwSize_p; wIndex++)
        // Anwenderdaten in Sendepuffer kopieren (Sendepuffer mu innerhalb 
        // eines Segmentes liegen, Anwenderdaten liegen irgendwo)
        abCommand_l[3 + wIndex] = pbSrcPtr_p[wIndex];

    // Ressource parametrieren: Sendepuffer, Empfangspuffer deklarieren
    C14SpiInitRessource (bEpromNr_p, (WORD)dwSize_p+3, abCommand_l, abCommand_l, NULL);
    // Transfer starten
    C14SpiStartSingleCycle(bEpromNr_p);

    // Status pollen: bertragung abgeschlossen?
    while (C14SpiGetRrcState(bEpromNr_p) == SPI_DEVICE_BUSY);
    
    //------------------------------------------------------------------
    // check if "write in process" is to end    
    SpiWipPoll(bEpromNr_p);                    
    // clear the write enable bit 
    SpiWrdi(bEpromNr_p);                       
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     SpiWren
//
// Description:  This function sets the "write enable" bit in the status
//               register.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWren(BYTE bEPROMNR_p) 
{
BYTE abCommand[] = {CMD_WREN};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);

    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);
}

//---------------------------------------------------------------------------
//
// Function:     SpiWrdi
//
// Description:  This function clears the "write enable" bit in the status
//               register.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWrdi(BYTE bEPROMNR_p)
{
BYTE abCommand[] = {CMD_WRDI};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);

    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);
}

//---------------------------------------------------------------------------
//
// Function:     SpiRdsr
//
// Description:  This function reads the status register of the spi device
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      Value of the status register
//
// State:        
//
//---------------------------------------------------------------------------
static BYTE SpiRdsr(BYTE bEPROMNR_p) 
{
BYTE abCommand[] = {CMD_RDSR, 0xFF};

    C14SpiInitRessource (bEPROMNR_p, tabentries (abCommand), 
                         abCommand, abCommand, NULL);
    C14SpiStartSingleCycle(bEPROMNR_p);

    while (C14SpiGetRrcState(bEPROMNR_p) == SPI_DEVICE_BUSY);

    return (abCommand[1]);
}

//---------------------------------------------------------------------------
//
// Function:     SpiWipPoll
//
// Description:  This function checks, if a write process in the eeprom is
//               ended.
//
// Parameters:   bEPROMNR_p : number of the spi device
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
static void SpiWipPoll(BYTE bEPROMNR_p)
{
BYTE bTmp;

    do 
    {
        bTmp = SpiRdsr(bEPROMNR_p);             // read the status register
        bTmp &= 0x01;                           // mask the WIP bit
    } 
    while (bTmp != WIPBIT);                     // repeat until WIP bit is "0"
}

// EOF
