/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  driver for the real time clock

                Notice: The driver includes more functions as supported as 
                        API functions. 
                        List of supported API functions:

                        - RtcInit
                        - RtcGetAbsTime
                        - RtcSetAbsTime
                        - RtcAbsToRelTime
                        - RtcRelToAbsTime
                        - RtcGetDaysOfMonth

  -------------------------------------------------------------------------

                $RCSfile: RtcDrv.c,v $
                
                $Author: K.Otto $
                
                $Revision: 1.2 $  $Date: 2004/09/20 13:03:50 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/03/10 -ct:   start of implementation

****************************************************************************/

#ifdef __C166__
    #include <xc161.h>
    #include <intrins.h>
#endif

#include "generic/global.h"
#include "../include/rtcdrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// alarm-interrupt will not be supported #define ALARM_INTERRUPT

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static DWORD dwRtcRelTimeSec_g;
static BYTE  bTimeValid_g;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------
static void I2cInit       (void);
static BYTE I2cWriteBlock (BYTE *pbSrc_p,  BYTE bDeviceID_p, 
                           BYTE bDestAddr_p, WORD wSize_p);
static BYTE I2cReadBlock  (BYTE *pbDest_p, BYTE bDeviceID_p, 
                           BYTE bSrcAddr_p,  WORD wSize_p);


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <rtc driver>                                        */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//  Public function for the access to the RTC.
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//                         Jan Feb Mrz Apr May Jun Jul Aug Sep Oct Nov Dec
const BYTE abDayTab_l[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

typedef struct 
{
    BCD8  Second;           
    BCD8  Minute;
    BCD8  Hour;
    BCD8  Day;
    BCD8  Weekday;
    BCD8  MonthCentury;
    BCD8  Years;

} tRtcTime;


typedef struct 
{
    BCD8  Minute;
    BCD8  Hour;
    BCD8  Day;
    BCD8  Weekday;

} tRtcAlarmTime;


//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//static tfpRtcCbHandler fpAppRtcHandler_l;
//static tfpRtcCbHandler fpAppRtcAlarmHandler_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static int   RtcBcd2Int      (BCD16 BCDValue_p);
static BCD16 RtcInt2Bcd      (int iValue_p);
//static void  RtcTimeHandler  (void);
//void PUBLIC  RtcAlarmHandler (void);

BYTE  PUBLIC RtcSetAlarmTime (tAlarmTime FAR* pAlarmTime_p);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     RtcInit
//
// Description:  This function initialize the I2C Interface and the RTC.
//
// Parameters:   none
//
// Returns:      error code
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcInit (void)
{
BYTE       bTxByte;
BYTE       bRtcAddr;
BYTE       bPowerFail;
BYTE       bErrorValue;
tRealTime  RealTime;
tAlarmTime AlarmTime;


    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P9:
    ///  -----------------------------------------------------------------------
    ///  P9.0 is used as general input
    ///  P9.1 is used as general input
    ///  P9.2 is used as general input
    ///  P9.3 is used as general input
    ///  P9.4 is used as general purpose output
    ///  - push/pull output is selected
    ///  - the pin status is high level
    ///  P9.5 is used as general purpose output
    ///  - push/pull output is selected
    ///  - the pin status is high level
    
    ///  P9.0 - P9.7 threshold type: TTL input
    
    ///  P9.0 - P9.3 output driver characteristic: strong driver
    ///  P9.4 - P9.7 output driver characteristic: strong driver
    
    ///  P9.0 - P9.3 output edge characteristic: sharp edge mode
    ///  P9.4 - P9.7 output edge characteristic: sharp edge mode
    
    ODP9          |=  0x0000;      // load open-drain register
    P9            |=  0x0030;      // load data register
    ALTSEL0P9     |=  0x0000;      // load alternate output function select 
                                   // register 0
    ALTSEL1P9     |=  0x0000;      // load alternate output function select 
                                   // register 1
    POCON9        |=  0x0000;      // load output control register
    DP9           |=  0x0030;      // load direction register
  
    I2cInit();                     // initialize the I2C interface

    // init local variables
    bErrorValue = RTC_SUCCESSFUL;
    
    RealTime.bSecond  = 0;
    RealTime.bMinute  = 0;
    RealTime.bHour    = 0;
    RealTime.bDay     = 1;
    RealTime.bWeekday = 7;
    RealTime.bMonth   = 1;
    RealTime.wYears   = 1980;

    AlarmTime.bMinute  = 0;
    AlarmTime.bHour    = 0;
    AlarmTime.bDay     = 0;
    AlarmTime.bWeekday = 0;

    // befor any first write acces, must read the VL bit, because after any write
    // acces this bi is CLEARED !!!
    // test for VL-Bit = "1" -> "power on" or "batterie low" detect
    bErrorValue = I2cReadBlock(&bPowerFail, RTC_ADDR, RTC_SECONDS_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {                                       
        goto EXIT;
    }

    // To set any Time, ClockOut, Alarmtime first hold on the rtc
    // set the values and then start the rtc again.
    bTxByte = 0x20;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    // new value for clkout frequency register
    bTxByte = 0x83;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CLKOUT_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }


    // start the RTC
    bTxByte = 0x00;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    // evaluate the VL bit :
    // VL-Bit  == 0 -> every thing is o.k.
    // VL-Bit  == 1 -> powerfail detect
    if((bPowerFail & 0x80) == 0x80)
    {
        // set start time
        RtcSetAbsTime (&RealTime);

        // set alarm time to 0
        RtcSetAlarmTime (&AlarmTime);

        // set the relative time to zero
        dwRtcRelTimeSec_g = 0;

        // set time in the rtc to invalid
        bTimeValid_g = FALSE;
    }
    else
    {
        RtcGetAbsTime (&RealTime);

        // encode the relative time
        dwRtcRelTimeSec_g = RtcAbsToRelTime (&RealTime);

        // set time in the rtc to valid
        bTimeValid_g = TRUE;
    }

    // Callback-Handler ungltig setzen
//    fpAppRtcHandler_l      = NULL;
//    fpAppRtcAlarmHandler_l = NULL;


    // initialize the RTC interrrupt input P9.3
    // to do if the interrupt is used

EXIT:


    return(bErrorValue);
}


//---------------------------------------------------------------------------
//
// Function:     RtcGetRelTime
//
// Description:  This function get the relative time in second since 
//               01.01.1980
//
// Parameters:   none
// 
// Returns:      relative time
//
// State:        
//
//---------------------------------------------------------------------------

DWORD PUBLIC RtcGetRelTime (void)
{
DWORD dwRetValue;


    _atomic_(0);
    dwRetValue = dwRtcRelTimeSec_g;
    _endatomic_();

    return(dwRetValue);

}


//---------------------------------------------------------------------------
//
// Function:     RtcSetRelTime
//
// Description:  This function set the relative time 
//
// Parameters:   relative time
// 
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC RtcSetRelTime (DWORD dwRelTime_p)
{
tRealTime RealTime;


    _atomic_(0);
    dwRtcRelTimeSec_g = dwRelTime_p;
    _endatomic_();

    // convert absolute time into relative time
    RtcRelToAbsTime (&RealTime, dwRelTime_p);

    // write the relative time into the RTC
    RtcSetAbsTime (&RealTime);

}


//---------------------------------------------------------------------------
//
// Function:     RtcSetAbsTime
//
// Description:  This function sets the new absolute time in the RTC,
//               converts it into a relative time and sets the variable for 
//               the relative Time.
//               Start of time is the 01.01.1980, 00:00
//
// Parameters:   pRealTime_p : pointer to the new time
//
// Returns:      error code
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcSetAbsTime (tRealTime FAR* pRealTime_p)
{
BYTE     bTxByte;
DWORD    dwValue;
BYTE     bErrorValue;
tRtcTime RtcTime;


    bErrorValue = RTC_SUCCESSFUL;

    // convert time and date in the BCD format
    RtcTime.Second       = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bSecond);
    RtcTime.Minute       = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bMinute);
    RtcTime.Hour         = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bHour  );

    RtcTime.Weekday      = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bWeekday);
    RtcTime.Day          = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bDay   );
    RtcTime.MonthCentury = (BCD8) RtcInt2Bcd ((int)pRealTime_p->bMonth );
    RtcTime.Years        = (BCD16) RtcInt2Bcd ((int)pRealTime_p->wYears );

    // set time in the rtc as valid
    bTimeValid_g = TRUE;
    pRealTime_p->bTimeValid = bTimeValid_g;

    // check for century bit to set
    if(pRealTime_p->wYears >= 2000)
    {
        RtcTime.MonthCentury |= 0x80;
    }

    // new value for control register to stop RTC & prepare setting of time
    bTxByte = 0x20;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    bErrorValue = I2cWriteBlock((BYTE*) &RtcTime, RTC_ADDR, RTC_SECONDS_REG, RTC_SIZE_OF_REAL_TIME);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    // new value for control register to start RTC
    bTxByte = 0x00;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;        
    }

    // encode the relative time
    dwValue = RtcAbsToRelTime (pRealTime_p);

    // set relative time
    _atomic_(0);
    dwRtcRelTimeSec_g = dwValue;
    _endatomic_();

EXIT:
    return(bErrorValue);

}


//---------------------------------------------------------------------------
//
// Function:     RtcGetAbsTime
//
// Description:  This function reads the actual absolute time out of the RTC
//
// Parameters:   pRealTime_p : pointer for the absolute time
// 
// Returns:      error code 
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcGetAbsTime (tRealTime FAR* pRealTime_p)
{
BYTE     bErrorValue;
tRtcTime RtcTime;


    bErrorValue = RTC_SUCCESSFUL;

    // read the actual absolute time out of the RTC
    bErrorValue = I2cReadBlock((BYTE*)&RtcTime, RTC_ADDR, RTC_SECONDS_REG, RTC_SIZE_OF_REAL_TIME);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;        
    }

    // clear unused bit
    RtcTime.Second  &= ~0x80;
    RtcTime.Minute  &=  0x7F;
    RtcTime.Hour    &=  0x3F;
    RtcTime.Weekday &=  0x07;
    RtcTime.Day     &=  0x3F;

    // convert time and date out of the BCD format
    pRealTime_p->bSecond  = (BYTE) RtcBcd2Int ((BCD16) RtcTime.Second );
    pRealTime_p->bMinute  = (BYTE) RtcBcd2Int ((BCD16) RtcTime.Minute );
    pRealTime_p->bHour    = (BYTE) RtcBcd2Int ((BCD16) RtcTime.Hour   );
    pRealTime_p->bWeekday = (BYTE) RtcBcd2Int ((BCD16) RtcTime.Weekday);
    pRealTime_p->bDay     = (BYTE) RtcBcd2Int ((BCD16) RtcTime.Day    );

    pRealTime_p->wYears   = (WORD) RtcBcd2Int ((BCD16) RtcTime.Years  );

    pRealTime_p->bTimeValid = bTimeValid_g;

    // check for century bit
    if((RtcTime.MonthCentury & 0x80) == 0x80)
    {
        pRealTime_p->wYears += 2000;
    }
    else
    {
        pRealTime_p->wYears += 1900;
    }

    RtcTime.MonthCentury &= 0x1F;
    pRealTime_p->bMonth = (BYTE)RtcBcd2Int((BCD16) RtcTime.MonthCentury);

EXIT:
    return(bErrorValue);

}


//---------------------------------------------------------------------------
//
// Function:     RtcClearAlarmFlag
//
// Description:  This function clear the alarm flag in the control2 register
//
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC RtcClearAlarmFlag(void)
{
BYTE bValue_p = 0x02;


    I2cWriteBlock(&bValue_p, RTC_ADDR, RTC_CNTRL2_REG, RTC_SIZE_OF_REG);

}


//---------------------------------------------------------------------------
//
// Function:     RtcGetAlarmFlag
//
// Description:  This function check the alarm flag in thr rtc
//
// Parameters:   none
//
// Returns:      TRUE  : alarm occur
//               FALSE : no alarm occur
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcGetAlarmFlag(void)
{
BYTE bCntrlValue;


    I2cReadBlock(&bCntrlValue, RTC_ADDR, RTC_CNTRL2_REG, RTC_SIZE_OF_REG);

    bCntrlValue &= 0x1F;

    if((bCntrlValue & 0x08) == 0x08)
    {
        return(TRUE);
    }
    else
    {
        return(FALSE);
    }

}


//---------------------------------------------------------------------------
//
// Function:     RtcSetAlarmTime
//
// Description:  This function sets the new alarm time in the RTC and 
//               activates the alarm for hours and minute
//
// Parameters:   pRealTime_p : pointer to the new alarm time
//
// Returns:      error code
//
// State:        
//
//---------------------------------------------------------------------------

BYTE  PUBLIC RtcSetAlarmTime (tAlarmTime FAR* pAlarmTime_p)
{
BYTE          bTxByte;
BYTE          bErrorValue;
tRtcAlarmTime AlarmTime;


    bErrorValue = RTC_SUCCESSFUL;

    // convert time and date in the BCD format
    AlarmTime.Minute       = (BCD8) RtcInt2Bcd ((int)pAlarmTime_p->bMinute);
    AlarmTime.Hour         = (BCD8) RtcInt2Bcd ((int)pAlarmTime_p->bHour  );
    AlarmTime.Weekday      = (BCD8) RtcInt2Bcd ((int)pAlarmTime_p->bWeekday);
    AlarmTime.Day          = (BCD8) RtcInt2Bcd ((int)pAlarmTime_p->bDay   );

    // activate the alarm		
    AlarmTime.Minute  &= ~0x80;
    AlarmTime.Hour    &= ~0x80;
    AlarmTime.Weekday |= 0x80;
    AlarmTime.Day     |= 0x80;

    // new value for control register to stop RTC & prepare setting of time
    bTxByte = 0x20;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    bErrorValue = I2cWriteBlock((BYTE*) &AlarmTime, RTC_ADDR, RTC_MINUTE_ALARM_REG, RTC_SIZE_OF_ALARM_TIME);
    if(bErrorValue != RTC_SUCCESSFUL)
    {
        goto EXIT;
    }

    // new value for control register to start RTC
    bTxByte = 0x00;
    bErrorValue = I2cWriteBlock(&bTxByte, RTC_ADDR, RTC_CNTRL1_REG, RTC_SIZE_OF_REG);

EXIT:
    return(bErrorValue);

}


//---------------------------------------------------------------------------
//
// Function:     RtcGetAlarmTime
//
// Description:  This function reads the alarm time out of the RTC
//
// Parameters:   pRealTime_p : pointer for the alarm time
// 
// Returns:      error code 
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcGetAlarmTime (tAlarmTime FAR* pAlarmTime_p)
{
BYTE          bErrorValue;
tRtcAlarmTime AlarmTime;


    bErrorValue = RTC_SUCCESSFUL;

    // read the actual alarm time out of the RTC
    bErrorValue = I2cReadBlock((BYTE*)&AlarmTime, RTC_ADDR, RTC_MINUTE_ALARM_REG, RTC_SIZE_OF_ALARM_TIME);
    if(bErrorValue != RTC_SUCCESSFUL )
    {
        goto EXIT;
    }

    // clear unused bit
    AlarmTime.Minute  &= 0x7F;
    AlarmTime.Hour    &= 0x3F;
    AlarmTime.Weekday &= 0x07;
    AlarmTime.Day     &= 0x3F;

    // convert time and date out of the BCD format
    pAlarmTime_p->bMinute  = (BYTE) RtcBcd2Int ((BCD16) AlarmTime.Minute );
    pAlarmTime_p->bHour    = (BYTE) RtcBcd2Int ((BCD16) AlarmTime.Hour   );
    pAlarmTime_p->bWeekday = (BYTE) RtcBcd2Int ((BCD16) AlarmTime.Weekday);
    pAlarmTime_p->bDay     = (BYTE) RtcBcd2Int ((BCD16) AlarmTime.Day    );

EXIT:
    return(bErrorValue);

}


//---------------------------------------------------------------------------
//
// Function:     RtcAbsToRelTime
//
// Description:  This function converts a absolute time into a relative time,
//               since 01.01.1980 00:00h
//
// Parameters:   pRealTime_p : pointer to absolute time structure
//
// Returns:      value of the relative time
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------
DWORD PUBLIC RtcAbsToRelTime (tRealTime FAR* pRealTime_p)
{
DWORD  dwRelTime;
BYTE   bTmp;


    // caculate the count of days since 01.01.1980 to 31.12. of the last complete year
    // (for every leap year on additiona day)
    dwRelTime = (pRealTime_p->wYears-1980)*365 + (pRealTime_p->wYears-(1980-3))/4;

    // add the count of days of complete month of the actual year
    for (bTmp = 1; bTmp < pRealTime_p->bMonth; bTmp++)
    {
        dwRelTime += RtcGetDaysOfMonth (bTmp, pRealTime_p->wYears);
    }

    // add complete days of the actual month
    dwRelTime += pRealTime_p->bDay-1;


    // the <dwRelTime> include the count of days since 01.01.1980 
    // this value is converted into seconds
    dwRelTime *= (DWORD)(24L*3600L);

    // add the sconds of the actual day since midnight
    dwRelTime += (DWORD)pRealTime_p->bHour*3600L;
    dwRelTime += (DWORD)pRealTime_p->bMinute*60L;
    dwRelTime += (DWORD)pRealTime_p->bSecond;

    return (dwRelTime);

}


//---------------------------------------------------------------------------
//
// Function:     RtcRelToAbsTime
//
// Description:  This function converts a relative time into a absolute time
//
// Parameters:   pRealTime_p : pointer to absolute time structure
//               dwRelTime_p : value of the relative time in seconds
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC RtcRelToAbsTime (tRealTime FAR* pRealTime_p, DWORD dwRelTime_p)
{
DWORD  dwDays;
DWORD  dwSubDays;
BYTE   bTmp;


    // convert the seconds in the count of days since 01.01.1980
    dwDays = dwRelTime_p / (DWORD)(24L*3600L);


    // after the substraction of the days, in <dwRelTime_p> stand the rest
    // of the seconds of the last started day
    dwRelTime_p -= (DWORD)(dwDays * (DWORD)(24L*3600L));

    // repeat substraction of 365 respectively 366 days, get the count 
    // of years since 1980
    bTmp=0;
    do
    {
        dwSubDays = 365L;
        if(bTmp%4 == 0)
        {
            dwSubDays++;
        }
        if(dwDays < dwSubDays)
        {
            break;
        }
        else
        {
            dwDays -= dwSubDays;
        }
        bTmp++;
    }
    while (1);

    pRealTime_p->wYears = 1980+bTmp;

    // find out through substraction the days of each month of the last 
    // complete month (with take into consideration of the February
    // in the leap year)
    bTmp=0;
    do
    {
        dwSubDays = (DWORD)(RtcGetDaysOfMonth(bTmp+1,pRealTime_p->wYears));
        if(dwDays < dwSubDays)
        {
            break;
        }
        else
        {
            dwDays -= dwSubDays;
        }
      bTmp++;
    }
    while (1);
    pRealTime_p->bMonth = bTmp+1;
    pRealTime_p->bDay   = (BYTE)dwDays+1;

    // divide the hole rest of <dwRelTime_p> into hour/minute/second
    // of the last stated day
    bTmp = (BYTE)(dwRelTime_p/3600L);
    dwRelTime_p -= (DWORD)bTmp * 3600L;
    pRealTime_p->bHour = bTmp;

    bTmp = (BYTE)(dwRelTime_p/60L);
    dwRelTime_p -= (DWORD)bTmp * 60L;
    pRealTime_p->bMinute = bTmp;

    pRealTime_p->bSecond = (BYTE)dwRelTime_p;
 
}


//---------------------------------------------------------------------------
//
// Function:     RtcInstallHandler
//
// Description:  This function installs a function pointer for the 
//               application. This function is called in "RtcTimeHandler"
//               depend on the clock out signal of the RTC.
//
// Parameters:   fpAppRtcHandler_p : function pointer of application
// 
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
/*
void PUBLIC RtcInstallHandler(tfpRtcCbHandler fpAppRtcHandler_p)
{

    // Interrupt sperren


    // Handler eintragen
    fpAppRtcHandler_l = fpAppRtcHandler_p;

    // Interrupt freigeben

  
}
*/

//---------------------------------------------------------------------------
//
// Function:     RtcInstallAlarmHandler
//
// Description:  This function installs a function pointer for the 
//               application. This function is called in 
//               "RtcAlarmHandler" when an alarm of the rtc occurs.
//
// Parameters:   fpAppRtcAlarmHandler_p : function pointer of applikation
// 
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
/*
void PUBLIC RtcInstallAlarmHandler(tfpRtcCbHandler fpAppRtcAlarmHandler_p)
{

    // Interrupt sperren
    

    // Handler eintragen
    fpAppRtcAlarmHandler_l = fpAppRtcAlarmHandler_p;

    // Interrupt freigeben
      
  
}

*/

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     RtcBcd2Int
//
// Description:  function to convert a BCD value into integer
//
// Parameters:   BCDValue_p : BCD value
//
// Returns:      integer value
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static int RtcBcd2Int (BCD16 BCDValue_p) 
{
int iResult;


    iResult = BCDValue_p & 0x000F;
    iResult += (((BCDValue_p & 0x00F0) >> 4) * 10);
    iResult += (((BCDValue_p & 0x0F00) >> 8) * 100);
    iResult += (((BCDValue_p & 0xF000) >> 12) * 1000);

    return(iResult);

}


//---------------------------------------------------------------------------
//
// Function:     RtcInt2Bcd
//
// Description:  function to convert a integer value to BCD
//
// Parameters:   iValue_p : interger Value
//
// Returns:      BCD16 Value
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BCD16 RtcInt2Bcd(int iValue_p)
{
BCD16 Result;


    Result = iValue_p % 10;
    iValue_p = iValue_p / 10;
    Result |= ((iValue_p % 10) << 4);
    iValue_p = iValue_p / 10;
    Result |= ((iValue_p % 10) << 8);
    iValue_p = iValue_p / 10;
    Result |= ((iValue_p % 10) << 12);

    return(Result);

}


//---------------------------------------------------------------------------
//
// Function:     RtcGetDaysOfMonth
//
// Description:  This function calculates the number of days of month
//
// Parameters:   bMonth : actual month
//               wYear  : actual year
//
// Returns:      days of month
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC RtcGetDaysOfMonth (BYTE bMonth, WORD wYear)
{
BYTE  bDays;


    bDays = abDayTab_l[bMonth-1];

    // check for leap year
    if( (bMonth == 2) && (wYear%4 == 0))
    {
        bDays++;
    }

    return (bDays);

}

//---------------------------------------------------------------------------
//
// Function:     RtcTimeHandler
//
// Description:  This is the ISR handler for the rtc clock out impulse.
//               The time base of clock out is second
//  
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
/*
static void RtcTimeHandler(void)
{

    // call handler from the application
    if(fpAppRtcHandler_l != NULL)
    {
        fpAppRtcHandler_l();
    }

}
*/

//---------------------------------------------------------------------------
//
// Function:     RtcAlarmHandler
//
// Description:  This is the ISR handler for the alarm interrupt of the RTC
//
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------
/*
void PUBLIC RtcAlarmHandler(void)
{

    // clear alarm bit in the RTC control register 2
    RtcClearAlarmFlag();

    // call handler from the application
    if(fpAppRtcAlarmHandler_l != NULL)
    {
        fpAppRtcAlarmHandler_l();
    }


}

*/
/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <I2C Interface>                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//  general functions for the I2C interface
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define  I2C_SUCCESSFUL                 0x00
#define  I2C_CROSSING_PAGE_BOUNDARIES   0xFC
#define  I2C_DATA_TO_LONG               0xFD
#define  I2C_NO_ACKNOWLEDGE             0xFE
#define  I2C_NOT_IMPLEMENTED            0xFF

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static void I2cWriteByte (BYTE bData_p);
static BYTE I2cReadByte  (void);
static void I2cStart     (void);
static void I2cStop      (void);
static void I2cSendAck   (void);
static BYTE I2cRecAck    (void);
static void I2cSendBit   (BYTE bState_p);
static BYTE I2cRecBit    (void);
static void I2cPutScl    (BYTE bState_p);
static BYTE I2cGetScl    (void);
static void I2cPutSda    (BYTE bState_p);
static BYTE I2cGetSda    (void);


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     I2cInit
//
// Description:  This function initializes the I2C interface
//
// Parameters:   none
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cInit(void)
{

    // re-enable SDA-line
    I2cPutScl(0);                   // preset SCL=0 to avoid accidential
                                       // transmission of START condition
                                       // by following SDA-transition

    I2cPutSda(1);                   // drive SDA to inactive state
    I2cPutScl(0);                   // set SCL for the first time
    I2cPutScl(1);                   // release SCL
    I2cPutScl(0);                   // set SCL for the second time
    I2cPutScl(1);                   // release SCL
    I2cPutScl(0);                   // set SCL for the third time

// I2C-bus is now going in the inactive state by the following commands

    I2cPutSda(1);                   // this is also the state after RESET
    I2cPutScl(1);                   // setup SCL for START condition
   
}


//---------------------------------------------------------------------------
//
// Function:     I2cWriteBlock
//
// Description:  This function writes a count of byte to a I2C device
//
// Parameters:   pbSrc_p     : pointer to the source data
//               bDeviceID_p : I2C address of the device
//               bDestAddr_p : start address in hte device
//               wSize_p     : count of bytes to write
//
// Returns:      error code
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cWriteBlock(BYTE *pbSrc_p, BYTE bDeviceID_p, BYTE bDestAddr_p, WORD wSize_p)
{
BYTE bErrorValue;


    bErrorValue = I2C_SUCCESSFUL;
    
    // send START condition
    I2cStart();                          

    // send ID with cleared RD/WR bit
    I2cWriteByte(bDeviceID_p & 0xFE);    

    // is device responding ?
    bErrorValue = I2cRecAck();
    if(bErrorValue != I2C_SUCCESSFUL)
    {                                       
        goto EXIT;
    }

    // write destination address
    I2cWriteByte(bDestAddr_p);           

    // is device responding ?
    bErrorValue = I2cRecAck();
    if(bErrorValue != I2C_SUCCESSFUL)
    {
        goto EXIT;
    }

    // loop until all bytes send
    while(wSize_p--)                        
    {
        I2cWriteByte(*pbSrc_p);
        pbSrc_p += 1;

        // is device responding ?
        bErrorValue = I2cRecAck();
        if(bErrorValue != I2C_SUCCESSFUL)
        {
            goto EXIT;
        }
        
    }

EXIT:   
    // send STOP condition
    I2cStop();                           
    
    return(bErrorValue);

}

//---------------------------------------------------------------------------
//
// Function:     I2cReadBlock
//
// Description:  This function reads a count of byte out of a I2C device
//
// Parameters:   pbDest_p    : pointer to the destination memory
//               bDeviceID_p : I2C address of the device
//               bSrcAddr_p  : start address in the device
//               wSize_p     : count of bytes to read
//
// Returns:      error code
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cReadBlock(BYTE *pbDest_p, BYTE bDeviceID_p, BYTE bSrcAddr_p, WORD wSize_p)
{
BYTE bErrorValue;


    bErrorValue = I2C_SUCCESSFUL;

    // send START condition
    I2cStart();                          

    // send ID with cleared RD/WR bit
    I2cWriteByte(bDeviceID_p & 0xFE);    

    // is device responding ?
    bErrorValue = I2cRecAck();
    if(bErrorValue != I2C_SUCCESSFUL)
    {
        goto EXIT;
    }

    // write source address
    I2cWriteByte(bSrcAddr_p);            

    // is device responding ?
    bErrorValue = I2cRecAck();
    if(bErrorValue != I2C_SUCCESSFUL)
    {
        goto EXIT;
    }

    // send START condition
    I2cStart();                          

    // send ID with set RD/WR bit
    I2cWriteByte(bDeviceID_p | 0x01);

    // is device responding ?
    bErrorValue = I2cRecAck();
    if(bErrorValue != I2C_SUCCESSFUL)
    {
        goto EXIT;
    }

    // loop until all data received
    while(wSize_p--)                        
    {
        // read data
        *(pbDest_p++) = I2cReadByte();   

        if(wSize_p)                         
        {
            // send ACK to slave except last time
            I2cSendAck();
        }
        else
        {
            I2cSendBit(1);
        }
    }

EXIT:
    // send STOP condition
    I2cStop();                          
   
    return(bErrorValue);

}


//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     I2cWriteByte
//
// Description:  This function writes one byte into a I2C device
//
// Parameters:   bData_p : data byte to send
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cWriteByte(BYTE bData_p)
{
BYTE bLoop;


    for(bLoop = 0; bLoop <= 7; bLoop++)
    {
        I2cSendBit((bData_p & 0x80) == 0x80);
        bData_p = (bData_p << 1);
    }

}


//---------------------------------------------------------------------------
//
// Function:     I2cReadByte
//
// Description:  This function reads one byte out of a I2C device
//
// Parameters:   none
//
// Returns:      data bytes of the device
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cReadByte(void)
{
BYTE bValue;
BYTE bLoop;


    bValue = 0;

    for(bLoop = 0; bLoop <= 7; bLoop++)
    {
        bValue = (bValue << 1);             // 1. shift doesn't affect (value=0)
        bValue = bValue | I2cRecBit();
    }

   return(bValue);

}


//---------------------------------------------------------------------------
//
// Function:     I2cStart
//
// Description:  This function generates the "start condition" on the I2C Bus
//
// Parameters:   none
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cStart(void)
{

    I2cPutSda(1);
    I2cPutScl(1);
    I2cPutSda(0);
    I2cPutScl(0);

}


//---------------------------------------------------------------------------
//
// Function:     I2cStop
//
// Description:  This function generates the "stop condition" on the I2C Bus
//
// Parameters:   none
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cStop(void)
{
    
    I2cPutScl(0);
    I2cPutSda(0);
    I2cPutScl(1);
    I2cPutSda(1);
    
}


//---------------------------------------------------------------------------
//
// Function:     I2cSendACK
//
// Description:  This function sends a acknowledge on the I2C Bus
//
// Parameters:   none
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cSendAck(void)
{

    I2cSendBit(0);

}


//---------------------------------------------------------------------------
//
// Function:     I2cRecAck
//
// Description:  This function receives a acknowledge on the I2C Bus
//
// Parameters:   none
//
// Returns:      error code
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cRecAck(void)
{

    if (I2cRecBit() == 0)
    {
        return(I2C_SUCCESSFUL);          // must be 0 if device responds
    }
    else
    {
        return(I2C_NO_ACKNOWLEDGE);
    }

}


//---------------------------------------------------------------------------
//
// Function:     I2cSendBit
//
// Description:  This function sends a bit on the I2C Bus
//
// Parameters:   bState_p : bit to send
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cSendBit(BYTE bState_p)
{
   
    I2cPutScl(0);
    I2cPutSda(bState_p);
    I2cPutScl(1);

    while(!I2cGetScl());            // wait until SCL-line is released

    I2cPutScl(0);

}


//---------------------------------------------------------------------------
//
// Function:     I2cRecBit 
//
// Description:  This function receives a bit fron the I2C Bus
//
// Parameters:   bState_p : bit to send
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cRecBit(void)
{
BYTE bState;


    I2cPutScl(0);
    I2cPutSda(1);
    I2cPutScl(1);

    while(!I2cGetScl());          // wait until SCL-line is released

    bState = I2cGetSda();         // read SDA state
    I2cPutScl(0);

    return(bState);

}


//---------------------------------------------------------------------------
//
// Function:     I2cPutScl
//
// Description:  This function sets the declaired state to the SCL line of
//               the I2C Bus
//
// Parameters:   bState_p : state for the SCL line
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cPutScl(BYTE bState_p)
{

    if(!bState_p)
    {
        _atomic_(0);

        RTC_PORT &= SCL_LOW_MASK;        // set SCL-line to ZERO
        RTC_DIR_PORT |= SCL_HIGH_MASK;   // set port-pin SCL for output mode
                                         // enable output AFTER setting of
                                         // correct level to avoid '1'-level
                                         // spikes
        _endatomic_();
    }
    else
    {
        _atomic_(0);

        RTC_DIR_PORT &= SCL_LOW_MASK;
        RTC_PORT |= SCL_HIGH_MASK;       // set SCL-line to ONE

        _endatomic_();
    }

}


//---------------------------------------------------------------------------
//
// Function:     I2cGetScl
//
// Description:  This function gets the actual state to the SCL line of
//               the I2C Bus
//
// Parameters:   none
//
// Returns:      state for the SCL line
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cGetScl(void)
{

    RTC_DIR_PORT &= SCL_LOW_MASK;      // set port-pin SCL for input mode

    if((RTC_PORT & SCL_HIGH_MASK))
    {
        return(1);                     // read SCL-line status
    }
    else
    {
        return(0);
    }

}


//---------------------------------------------------------------------------
//
// Function:     I2cPutSda
//
// Description:  This function sets the declaired state to the SDA line of
//               the I2C Bus
//
// Parameters:   bState_p : state for the SDA line
//
// Returns:      none
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static void I2cPutSda(BYTE bState_p)
{

    if(!bState_p)
    {
        _atomic_(0);

        RTC_PORT &= SDA_LOW_MASK;        // set SDA-line to ZERO
        RTC_DIR_PORT |= SDA_HIGH_MASK;   // set port-pin SDA for output mode
                                       // enable output AFTER setting of
                                       // correct level to avoid '1'-level
                                       // spikes
        _endatomic_();
    }
    else
    {
        _atomic_(0);

        RTC_DIR_PORT &= SDA_LOW_MASK;    // set port-pin SDA for input mode
        RTC_PORT |= SDA_HIGH_MASK;       // set SDA-line to ONE

        _endatomic_();
    }

}


//---------------------------------------------------------------------------
//
// Function:     I2cGetSda
//
// Description:  This function gets the actual state to the SDA line of
//               the I2C Bus
//
// Parameters:   none
//
// Returns:      state for the SDA line
//
// State:        tested (18.06.2004 A.S.)
//
//---------------------------------------------------------------------------

static BYTE I2cGetSda(void)
{

    RTC_DIR_PORT &= SDA_LOW_MASK;       // set port-pin SDA for input mode

    if((RTC_PORT & SDA_HIGH_MASK))
    {
        return(1);                       // read SDA-line status
    }
    else
    {
        return(0);
    }

}


// EOF


