/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      COMBImodul-C14

  Description:  example for using hardware driver

                For running example connect serial interface ASC0 of
                the COMBImodul-C14 with an COM-port of PC. Select the 
                following parameters for the COM-port: 8 data bits, 1 stop 
                bit, non parity check, non protocoll.
                After loading and starting example program type 'help'
                to get a list of available commands. 

  -------------------------------------------------------------------------

                $RCSfile: TestFrame.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.9 $  $Date: 2005/01/04 10:15:59 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/08/13 -as:   Start der Implementierung

****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <intrins.h>

#include <XC161.h>
#include "generic/global.h"
#include "generic/gmacros.h"
#include "../include/cmdint.h"
#include "../../../include/timerdrv.h"

// hardware specific definitions
#include "../../../include/C14Drv.h"
#include "../../../include/eeprmdrv.h"
#include "../../../include/rtcdrv.h"
#include "../../../include/siodrv.h"
#include "../../../include/Pwrfdrv.h"
#include "../../../include/Rtos.h"
#include "../../../include/MdvSeDrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

DWORD PUBLIC htoul (char *str);

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Application Test>                                  */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse stellt eine MAIN-Routine zur Verfgung die als Test-
// Applikation verwendet werden kann.
//
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
typedef struct
{
    DWORD   m_dwDIN;
    long    m_liCnt[4];
    WORD    m_wAdc[4];

} tPIN;

typedef struct
{
    DWORD   m_dwDOUT;
    WORD    m_wAout[2];

} tPOUT;

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static tPIN  ImageInput_l;
static tPOUT ImageOutput_l;

static tPIN  *pImageInput_l;
static tPOUT *pImageOutput_l;

static BYTE bWatchdogEvent_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

void PUBLIC TestCmdInit   (void);
void PUBLIC TestImageInit (void *pInputImage_p, void *pOutputImage_p);
void PUBLIC TestImageGet  (void);
void PUBLIC TestImageSet  (void);
void PUBLIC TestRtosInit  (void);
void PUBLIC TestPowerFailHandler(bit bitPowerFailState);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------

void main (void)
{
    PSW_IEN = 1; 

    StartTimer();

    PowerFailInit ();
    
    PowerFailInstallHandler(TestPowerFailHandler);

    // Initialization of port pins
    C14Initialize ();
        
    TestImageInit(&ImageInput_l, &ImageOutput_l);
    TestCmdInit ();

    C14ResetWatchdog ();

    printf("\n\r> ");

    // Zyklusschleife   
    while(1)
    {   
        C14ResetWatchdog ();
        RtosResetWatchdog();

        TestImageGet();
        CmdIntProcess();
        TestImageSet();
    }
} 

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test Monitor>                                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
// Auswahl der zu untersttzenden Komponenten
#ifndef SIMULATION
  #define TEST_RTC_SUPPORT
#endif

#define TEST_EEPROM_SUPPORT
// #define TEST_MODEM_SUPPORT

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Klasse "Generic"
static void TestReset           (char ** pArgList_p);
static void TestMemDumpInit     (void);
static void TestMemDumpGet      (char ** pArgList_p);

//---------------------------------------------------------------------------
// Klasse "LZS Peripherie"
static void TestVersionGet      (char ** pArgList_p);
static void TestLedSet          (char ** pArgList_p);
static void TestGetLzsInfo      (char ** pArgList_p);

//---------------------------------------------------------------------------
// Klasse "EEPROM"
#ifdef TEST_EEPROM_SUPPORT
    static void TestEeprmInit   (void);
    static void TestEeprmClr    (char ** pArgList_p);
    static void TestEeprmGet    (char ** pArgList_p);
    static void TestEeprmSet    (char ** pArgList_p);
#endif

//---------------------------------------------------------------------------
// Klasse "Prozess Peripherie"
static void TestAdGet           (char ** pArgList_p);
static void TestDiGet           (char ** pArgList_p);
static void TestDoGet           (char ** pArgList_p);
static void TestDoSet           (char ** pArgList_p);
static void TestSetPort         (char ** pArgList_p);
static void TestGetPort         (char ** pArgList_p);
static void TestPwmInit         (void);
static void TestPwmSet          (char ** pArgList_p);
static void TestPtoSet          (char ** pArgList_p);
static void TestPtoPwmTbSet     (char ** pArgList_p);

//---------------------------------------------------------------------------
// Klasse "RTC"
#ifdef TEST_RTC_SUPPORT
    static void TestRtcInit         (void);
    static void TestRtcGetTimeDate  (char ** pArgList_p);
    static void TestRtcSetTimeDate  (char ** pArgList_p);
#endif

//---------------------------------------------------------------------------
// Klasse Modemsteuerleitungen
#ifdef TEST_MODEM_SUPPORT
    static void TestModemInit        (void);
    static void TestRtsSet           (char ** pArgList_p);
    static void TestDtrSet           (char ** pArgList_p);
    static void TestCtrGet           (char ** pArgList_p);
#endif

//---------------------------------------------------------------------------
// Kommandotabelle
//---------------------------------------------------------------------------
// Tabelle enthlt alle durch den Monitor untersttzte Kommandos

const tCmdTableEntry aCmdTable_l[] =
{
    // Klasse "Generic"
    {COMMAND("RESET"), "                   Reset\r\n",                       NULL,  TestReset},
    {COMMAND("DUMP "), "<Adr>              Dump memory\r\n",                 NULL,  TestMemDumpGet},
    {COMMAND("DUMP"), "                    Dump memory\r\n",      TestMemDumpInit,  TestMemDumpGet},
    // Klasse "LZS Peripherie"
    {COMMAND("VERSION"), "                 Display version\r\n",             NULL,  TestVersionGet},
    {COMMAND("LZS?"), "                    Display LZS-INFO\r\n",            NULL,  TestGetLzsInfo},

#ifdef TEST_MODEM_SUPPORT
    // Klasse "Modem"
    {COMMAND("MODEM?"), "                  Modem control lines\r\n",TestModemInit,  TestCtrGet},
    {COMMAND("RTS "), "<state>             Set RTS line\r\n",                NULL,  TestRtsSet},
    {COMMAND("DTR "), "<state>             Set DTR line\r\n",                NULL,  TestDtrSet},
#endif 

    // Klasse "Prozess Peripherie"
    {COMMAND("LED "), "<mask>              Set Run/Stop-LEDs\r\n",              NULL,  TestLedSet},
    {COMMAND("AD?"), "                     Display AI0 .. 3\r\n",            NULL,  TestAdGet},
    {COMMAND("DI?"), "                     Display DI0 .. 23\r\n",           NULL,  TestDiGet},
    {COMMAND("DO?"), "                     Display DO0 .. 19\r\n",           NULL,  TestDoGet},
    {COMMAND("SDO "), "<val>               Set     DO0 .. 19\r\n",           NULL,  TestDoSet},
    {COMMAND("SDO"), "                     Rotate  DO0 .. 19\r\n",           NULL,  TestDoSet},
    {COMMAND("SPWM "), "<ch per val>       Set    PWM0 .. 3\r\n",     TestPwmInit,  TestPwmSet},
    {COMMAND("SPTO "), "<ch prd val pls>   Set    PTO0 .. 1\r\n",            NULL,  TestPtoSet},
    {COMMAND("STB "), "<ch val>            Set Timer Base 0/1\r\n",          NULL,  TestPtoPwmTbSet},
    {COMMAND("SP "), "<port mask val>      Set Port Pins [P2, P9]\r\n",      NULL,  TestSetPort},
    {COMMAND("GP?"), "                     Get Port Pins\r\n",               NULL,  TestGetPort},

    // Klasse "EEPROM"
#ifdef TEST_EEPROM_SUPPORT
    {COMMAND("EEP? "), "<id> <off>         EEPROM read\r\n",                 NULL,  TestEeprmGet},
    {COMMAND("EEP?"), "                    EEPROM read\r\n",                 NULL,  TestEeprmGet},
    {COMMAND("SEEP "), "<id> <off> <data>  EEPROM write\r\n",                NULL,  TestEeprmSet},
    {COMMAND("CEEP"), "                    EEPROM clear\r\n",       TestEeprmInit,  TestEeprmClr},
#endif
    // Klasse "RTC"
#ifdef TEST_RTC_SUPPORT
    {COMMAND("RTC?"), "                    Display date, time\r\n",   TestRtcInit,  TestRtcGetTimeDate},
    {COMMAND("SRTC "), "<hhmmss ddmmyyyy>  Set date, time\r\n",              NULL,  TestRtcSetTimeDate},
#endif
    {COMMAND("HELP"), "                    Display commands\r\n",            NULL,  NULL},
    {COMMAND("Esc-Taste"), "               Repeat last command\r\n",         NULL,  NULL}
};


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestCmdInit
//
// Description:  Die Funktion init. statische Variablen der Klasse und
//               parametrierte den Kommando-Interpreter.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        getestet (12.03.2003 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC TestCmdInit (void)
{
    // Diese Funktion mu noch vor der ersten Zeichenausgabe 
    // ausgefhrt werden.
    CmdIntInit (aCmdTable_l, tab_entries(aCmdTable_l));

    CmdInitCommand();
}

//---------------------------------------------------------------------------
//
// Function:     TestImageInit
//
// Description:  Die Funktion init. das Prozessabbild.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC TestImageInit (void *pInputImage_p, void *pOutputImage_p)
{
WORD i;

    // Pointer auf Prozessabbild bekanntgeben
    pImageInput_l = pInputImage_p;
    pImageOutput_l= pOutputImage_p;

    // Prozessabbild init.  
    pImageInput_l->m_dwDIN   = 0;

    // Zhlereingnge
    for (i=0; i < tab_entries(pImageInput_l->m_liCnt); i++)
    {
        pImageInput_l->m_liCnt[i]   = 0;
        // Zhlregister init. 
        C14CntSet(i, 0x0L);
        // Zhler starten: steigende Flanke, externe Steuerung Auf/Abwrtszhlen
        // Soll die Steuerung der Richtung durch ein externes Port Pin erfolgen,
        // dann kann durch Setzen des Richtungsflags der aktive Pegel negiert werden.
        // COUNTING_DOWN: Eingang low  --> counting down
        //                Eingang high --> counting up
        // COUNTING_UP:   Eingang low  --> counting up
        //                Eingang high --> counting down
        C14CntSetMode (i, ANY_EDGE, EXT_UP_DOWN_CONTROL, COUNTER_RUN);
    }

    // Analoge Eingnge
    for (i=0; i < tab_entries(pImageInput_l->m_wAdc); i++)
        pImageInput_l->m_wAdc[i]   = 0;

    // Digitale Ausgnge
    pImageOutput_l->m_dwDOUT = 0;

    // Analoge Ausgnge
    pImageOutput_l->m_wAout[0] = 0;
    pImageOutput_l->m_wAout[1] = 0;

}

//---------------------------------------------------------------------------
//
// Function:     TestImageGet
//
// Description:  Die Funktion hinterlegt die Eingnge im Prozessabbild.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC TestImageGet (void)
{
WORD i;

    // Lesen der Eingnge und Schreiben Prozessabbild
    C14GetInGroup((BYTE FAR*)&ImageInput_l.m_dwDIN, 4);

    // Lesen analoge Eingnge und Schreiben Prozessabbild
    C14AdcGet ((BYTE FAR*)&ImageInput_l.m_wAdc[0], 8);

    // Lesen Zhlereingnge
    for (i=0; i < tab_entries(ImageInput_l.m_liCnt); i++)
        ImageInput_l.m_liCnt[i] = C14CntGet(i);

}

//---------------------------------------------------------------------------
//
// Function:     TestImageSet
//
// Description:  Die Funktion bertrgt das Prozessabbild an die Ausgnge.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC TestImageSet (void)
{
    // Lesen Prozessabbild und Schreiben der Ausgnge
    C14SetOutGroup((BYTE FAR*)&ImageOutput_l.m_dwDOUT, 3);

    C14AoutSet((BYTE FAR*)&ImageOutput_l.m_wAout[0], sizeof(ImageOutput_l.m_wAout));
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

// ...


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test Generic>                                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Klasse definierte allgemeingltige Funktionen fr die Untersttzung
// von Test. 
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

// Ablage letzte Adresse Speicherdump
static DWORD dwMemDumpAddr_l;
// Zwischenspeicher fr Memory Dump
static char  ASCII[16+1];

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestReset
//
// Description:  Die Funktion lst Software-Reset aus.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        getestet (12.03.2003 A.S.)
//
//---------------------------------------------------------------------------

static void TestReset    (char ** pArgList_p)
{
// Geht nicht, keine Ahnung warum.   _trap_(0);
    printf("\r\n\r\nWaiting for reset target ...");
    // Watchdog enablen und warten bis Watchdog RESET auslst
//    DP3 |= 0x0004;      // enable output driver = Watchdog enable
_trap_(0);
    while(1);    
}

//---------------------------------------------------------------------------
//
// Function:     TestMemDumpInit
//
// Description:  Die Funktion init. lokale Variablen fr Memory-Dump.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        getestet (12.03.2003 A.S.)
//
//---------------------------------------------------------------------------

static void TestMemDumpInit (void)
{

    dwMemDumpAddr_l = 0L;
}

//---------------------------------------------------------------------------
//
// Function:     TestMemDump
//
// Description:  Die Funktion gibt ab Addr. den Speicherbereich als HEX-
//               DUMP aus (HEX + ASCII - Darstellung). Der Ausgabebereich 
//               ist auf 256 Zeichen begrenzt.
//
//
// Parameters:   Startadresse fr dioe Ausgabe
//
// 
// Returns:      keine 
//
//
// State:        getestet (12.03.2003 A.S.)
//
//---------------------------------------------------------------------------

static void TestMemDump (DWORD dwAddr_p)
{
BYTE GENERIC *pMem;   
WORD         i;      
WORD         k;  
BYTE         cZeichen;

    pMem = (BYTE GENERIC*)dwAddr_p;
    printf("\r\n");

    k = 0;
    for (i=0; i<256;)                   // 256 Bytes anzeigen ... 
    {
        if (!(i%16))                    // Bereits 16 Bytes angezeigt ? 
        {
           printf("%08lX: ", (DWORD)(dwAddr_p+i)); // Adresse anzeigen
        }

        cZeichen = pMem[i];             // Zeichen lesen

        printf("%02bX ", cZeichen);     // Byte in Hex anzeigen 

        if ( (cZeichen >= ' ') && 
             (cZeichen < 0x7F) )        // Byte als ASCII darstellbar ?
        {                          
           ASCII[k++] = cZeichen;       // Byte in ASCII-String uebernehmen 
        }
        else                            // NEIN! Byte nicht darstellbar!
        {
           ASCII[k++] = '.';            // Durch Sonderzeichen ersetzen
        }
        ASCII[k] = '\0';                // Stringende-Kennzeichen setzen

        i++;
        // ASCII-Zeichen ausgeben ? 
        if (!(i%16))                    // Bereits 16 Bytes angezeigt ? 
        {                                
           printf("    ");              // Abstand zur ASCII-Darstellung
           printf("%s",&ASCII[0]);      // ASCII-Darstellung ausgeben 
           printf("\r\n"); 
           k = 0;                       // Wieder an Anfang von ASCII[]
        }
    }
}

//---------------------------------------------------------------------------
//
// Function:     TestMemDumpGet
//
// Description:  Die Funktion erzeugt ein Mem-Dump.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        getestet (12.03.2003 A.S.)
//
//---------------------------------------------------------------------------

static void TestMemDumpGet (char ** pArgList_p)
{
    // DUMP-Kommando ohne Adresse ?
    if (!strcmp(pArgList_p[0],"DUMP"))
    {
       TestMemDump (dwMemDumpAddr_l); // MemDump ab aktueller Adresse   
       dwMemDumpAddr_l += 256;        // Addresse weiterstellen        
    }

    // Dump-Kommando (mit Adresse) ? 
    if (!memcmp(pArgList_p[0],"DUMP ",5))
    {
       dwMemDumpAddr_l = htoul(pArgList_p[1]);  // Adresse holen 
       TestMemDump (dwMemDumpAddr_l);           // MemDump ab aktueller Adresse 
       dwMemDumpAddr_l += 256;                  // Addresse weiterstellen
    }

    printf("\r\n");
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test LZS Peripherie>                               */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestVersionGet
//
// Description:  Die Funktion gibt die aktuelle Softwareversion aus.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestVersionGet (char ** pArgList_p)
{
    printf("\r\nSYSTEC electronic GmbH");
    printf("\r\nTESTPROGRAMM HARDWARE-TREIBER");
    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestLedSet
//
// Description:  Die Funktion schaltet die Status-LEDs auf der CPU-Platine.
//
//
// Parameters:   Pointer auf Argumentenliste
//
//               Bit0: RUN-LED
//               Bit1: ERROR-LED
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestLedSet (char ** pArgList_p)
{
BYTE bLedState;

    // Argument vorhanden ?
    if (!pArgList_p[1])
    {
       printf("LED x: fehlendes Argument\r\n");
    }
    else
    {
       bLedState = (BYTE)htoul(pArgList_p[1]);

       // LED_RUN, LED_ERROR schalten
       C14SetRunLED((bLedState & 0x01)?ON:OFF);
       C14SetErrLED((bLedState & 0x02)?ON:OFF);
    }

    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestGetLzsInfo
//
// Description:  Die Funktion ermittelt den aktuellen Zustand verschiedener
//               Control-Elemente der Baugruppe und gibt diese ber RS-232 aus.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestGetLzsInfo (char ** pArgList_p)
{
BYTE       bState;
tC14HwInfo HwInfo;

    bState = C14GetSwitch();
    if (bState == SWITCH_RUN)
    {
       printf("\r\nRUN/STOP/MRES = %s", "RUN");
    }
    else if (bState == SWITCH_STOP)
    {
       printf("\r\nRUN/STOP/MRES = %s", "STOP");
    }
    else if (bState == SWITCH_MRES)
    {
       printf("\r\nRUN/STOP/MRES = %s", "MRES");
    }
    else
    {
       printf("\r\nRUN/STOP/MRES = %s", "ERROR");
    }
    printf("\r\nDIP-Switch    = %02bX", C14GetDIPSwitch());
    printf("\r\nHEX-Switch    = %02bX", C14GetHexNumber());
    printf("\r\nIO-HW-CFG     = %02bX", C14GetIoHWCfg());
    printf("\r\nCPU-HW-CFG    = %02bX", C14GetCpuHWCfg());
    printf("\r\nBattery       = %s",    C14GetBattState()==TRUE?"ok":"wrong");
    printf("\r\nADCE          = %04X",  ADC_EIC);
    printf("\r\nSPIE          = %04X",  SSC0_EIC);
    printf("\r\nRSTCFG        = %04X",  RSTCFG);
    printf("\r\nRSTCON        = %04X",  RSTCON);
    printf("\r\nPLLCON        = %04X",  PLLCON);
    printf("\r\nEBCMOD0       = %04X",  EBCMOD0);
    printf("\r\nEBCMOD1       = %04X",  EBCMOD1);
    printf("\r\nADDRSEL1      = %04X",  ADDRSEL1);
    printf("\r\nTCONCS1       = %04X",  TCONCS1);
    printf("\r\nADDRSEL2      = %04X",  ADDRSEL2);
    printf("\r\nTCONCS2       = %04X",  TCONCS2);
    printf("\r\nADDRSEL3      = %04X",  ADDRSEL3);
    printf("\r\nTCONCS3       = %04X",  TCONCS3);
    printf("\r\nADDRSEL4      = %04X",  ADDRSEL4);
    printf("\r\nTCONCS4       = %04X",  TCONCS4);

    C14GetHardwareInfo (&HwInfo);
    printf("\r\nCPU-PCB       = %d.%d",HwInfo.m_wPcbCpuVersion, HwInfo.m_bPcbCpuRevision);
    printf("\r\nIO-PCB        = %d.%d",HwInfo.m_wPcbIoVersion, HwInfo.m_bPcbIoRevision);

    if (bWatchdogEvent_l)
    {
       printf("\n\nWatchdog Timeout\n\n");
       bWatchdogEvent_l = FALSE;
       P3 |= 0x0008;
       RtosResetWatchdog();
    }

    printf("\r\n");
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test EEPROM>                                       */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse untersttzt den Test des EEPROMs durch Funktionen zum 
// Lesen und Schreiben eines Speicherbereichs im EEPROM.
//
/***************************************************************************/

#ifdef TEST_EEPROM_SUPPORT


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define SIZE_CFG_DATA   0x80
#define SIZE_PROC_DATA  0x100

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

// Ablage letzte Adresse und Device-Nummer EEPROM-Dump
static DWORD     dwEeprDumpAddr_l;
static char      abDaten_l[16];
static tMdvSeCfg DeviceCfg_l;
static BYTE      wDeviceNo_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestEeprmInit
//
// Description:  Die Funktion init. lokale Variablen fr den EEPROM-Test.
//
//
// Parameters:   keine
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEeprmInit  (void)
{
tEprmInitCfg   EprmInitCfg;

    // Interface for EEPROM 1 init.
    EprmInitCfg.m_bDeviceType    = EPRM25256;
    EprmInitCfg.m_dwCfgSectSize  = SIZE_CFG_DATA;
    EprmInitCfg.m_dwPdatSectSize = SIZE_PROC_DATA;
    EprmInit (&EprmInitCfg);

    // Interface for EEPROM 2 init.
    DeviceCfg_l.m_bType     = EPRM25160;
    DeviceCfg_l.m_bDeviceID = SPI_DEVICE_EEPROM2;
    DeviceCfg_l.m_bBaudRate = 0;
    MdvSeInit (&DeviceCfg_l);

    dwEeprDumpAddr_l  = 0L;
    wDeviceNo_l       = 0;
}

//---------------------------------------------------------------------------
//
// Function:     TestEeprmWriteBlock
//
// Description:  The function writes an data block to the eeprom.
//
//
// Parameters:   eeprom id
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEeprmWriteBlock (BYTE bId_p, BYTE GENERIC *pbData_p, 
                                 DWORD dwEeprmAdr_p, DWORD dwSize_p)
{
DWORD dwStartTime; 

    PowerFailStartCriticalSection();    
    if (bId_p == 0)
    {
       EprmSaveData (pbData_p, dwEeprmAdr_p, dwSize_p);
    }
    else
    {
       MdvSeWriteData (pbData_p, dwEeprmAdr_p, dwSize_p);
    }
    PowerFailEndCriticalSection();    
}

//---------------------------------------------------------------------------
//
// Function:     TestEeprmReadBlock
//
// Description:  The function reads an data block from the eeprom.
//
//
// Parameters:   eeprom id
//
// 
// Returns:      error code
//
//
// State:        
//
//---------------------------------------------------------------------------

static BYTE TestEeprmReadBlock (BYTE bId_p, BYTE GENERIC *pbData_p, 
                                 DWORD dwEeprmAdr_p, DWORD dwSize_p)
{
DWORD dwSize;

    if (bId_p == 0)
    {
       return (EprmRecoverData (pbData_p, dwEeprmAdr_p, dwSize_p));
    }
    else
    {
       dwSize = MdvSeReadData (pbData_p, dwEeprmAdr_p, dwSize_p);
       if (!dwSize || (dwSize == -1) )
          return FALSE;
       else
          return TRUE;
    }
}

//---------------------------------------------------------------------------
//
// Function:     TestEeprmClr
//
// Description:  Die Funktion lscht einen EEPROM.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEeprmClr (char ** pArgList_p)
{
BYTE   abFillBytes[8+1];
WORD   wDeviceNo;
DWORD  dwSize;
DWORD  dwEeprmAdr;
WORD   i;
DWORD  dwProgDaten;
tEprmSectSize  EprmSectSize;

// Um Fehlermeldung im Simulator zu unterdrcken, wird das
// Feld Fillbyte ein Byte grer definiert. Anderenfalls 
// erfolgt beim Schreiben des 8. Bytes ein Lesezugriff auf
// den nchsten Wert, der Simulator bricht hier dann mit einer
// Access Violation ab.
DWORD  dwEntriesFillBytes = tab_entries (abFillBytes) - 1;

    if (!pArgList_p[1])
    {
       printf("EEPROM clear: Argument missing\r\n");
       return;
    }

    wDeviceNo = (BYTE)htoul(pArgList_p[1]);
    if (wDeviceNo > 1)
    {
       printf("EEPROM clear: Unvalid Device number\r\n");
       return;
    }

    printf("\r\n");

    // Der EEPROM wird gelscht, in dem ein Feld mit
    // 0xFF geschrieben wird.
    for (i=0; i < dwEntriesFillBytes; i++)
        abFillBytes[i] = 0xFF;

    if (wDeviceNo == 0)
    {
       EprmGetSectSize(&EprmSectSize);
       dwSize      = EprmSectSize.m_dwProgSectSize;
    }
    else
    {
      dwSize = DeviceCfg_l.m_dwEndAdr;
    }
    dwEeprmAdr  = 0L;
    i           = 0;

    while (dwSize)
    {
        if (dwSize < dwEntriesFillBytes)
        {
           TestEeprmWriteBlock (wDeviceNo, abFillBytes, dwEeprmAdr, dwSize);
           break;
        }

        // Funktion lscht Pageweise einen EEPROM
        TestEeprmWriteBlock (wDeviceNo, abFillBytes, dwEeprmAdr, dwEntriesFillBytes);

        dwEeprmAdr += dwEntriesFillBytes;
        dwSize     -= dwEntriesFillBytes;
        i++;

        if (!(i%100))
        {
           printf(".");
        }
    }

    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestEepromDump
//
// Description:  Die Funktion gibt ab Addr. den Speicherbereich als HEX-
//               DUMP eines EEPROMs aus (HEX + ASCII - Darstellung). Der 
//               Ausgabebereich ist auf 256 Zeichen begrenzt.
//
//
// Parameters:   EEPROM-Nummer
//               Startadresse fr die Ausgabe
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEepromDump (WORD wDeviceNo_p, DWORD dwOffset_p)
{
BYTE GENERIC *pMem;   
WORD          i;      
WORD          k;  
BYTE          cZeichen;

    if ( wDeviceNo_p > 1 ) 
    {
       printf("EEPROM: Unvalid Device number\r\n");
       return;
    }

    pMem = (BYTE GENERIC*)dwOffset_p;
    printf("\r\n");

    k = 0;
    for (i=0; i<256;)                   // 256 Bytes anzeigen ... 
    {
        if (!(i%16))                    // Bereits 16 Bytes angezeigt ? 
        {
           printf("%08lX: ", (DWORD)(dwOffset_p+i));// Adresse anzeigen

           // und die nchsten 16 Zeichen lesen
           if (!TestEeprmReadBlock (wDeviceNo_p, abDaten_l, (DWORD)pMem+i, 16))
           {
              // EEPROM vollstndig gelesen
              break;
           }
        }

        cZeichen = abDaten_l[i%16];

        printf("%02bX ", cZeichen);     // Byte in Hex anzeigen 

        if ( (cZeichen >= ' ') && 
             (cZeichen < 0x7F) )        // Byte als ASCII darstellbar ?
        {                          
           ASCII[k++] = cZeichen;       // Byte in ASCII-String uebernehmen 
        }
        else                            // NEIN! Byte nicht darstellbar!
        {
           ASCII[k++] = '.';            // Durch Sonderzeichen ersetzen
        }
        ASCII[k] = '\0';                // Stringende-Kennzeichen setzen

        i++;
        // ASCII-Zeichen ausgeben ? 
        if (!(i%16))                    // Bereits 16 Bytes angezeigt ? 
        {                                
           printf("    ");              // Abstand zur ASCII-Darstellung
           printf("%s",&ASCII[0]);      // ASCII-Darstellung ausgeben 
           printf("\r\n"); 
           k = 0;                       // Wieder an Anfang von ASCII[]
        }
    }
}


//---------------------------------------------------------------------------
//
// Function:     TestEeprmGet
//
// Description:  Die Funktion liest 256 Bytes aus einen EEPROM.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEeprmGet (char ** pArgList_p)
{
    // DUMP-Kommando ohne Adresse ?
    if (!strcmp(pArgList_p[0],"EEP?"))
    {
       TestEepromDump (wDeviceNo_l, dwEeprDumpAddr_l);
       dwEeprDumpAddr_l += 256;  // Addresse weiterstellen        
    }

    // Dump-Kommando (mit Adresse) ? 
    if (!memcmp(pArgList_p[0],"EEP? ",5))
    {
       if (!pArgList_p[1] || !pArgList_p[2])
       {
          printf("EEPROM read: Argument missing\r\n");
       }
       else
       {
          wDeviceNo_l = (BYTE)htoul(pArgList_p[1]);
          // Startadresse (Offset) lesen
          dwEeprDumpAddr_l = htoul(pArgList_p[2]);

          TestEepromDump (wDeviceNo_l, dwEeprDumpAddr_l);
          dwEeprDumpAddr_l += 256;  // Addresse weiterstellen
       }
    }

    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestEepromWrite
//
// Description:  Die Funktion schreibt ab Addr. den Datenstring in den 
//               ausgewhlten EEPROM.
//
//
// Parameters:   EEPROM-Nummer
//               Startadresse fr die Ausgabe
//               Datenstring
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEepromWrite (WORD wDeviceNo_p, DWORD dwOffset_p, char *pstrDaten_p)
{
WORD wSize;

    if ( wDeviceNo_p > 1 )
    {
       printf("EEPROM: Unvalid Device number\r\n");
       return;
    }

    wSize = strlen (pstrDaten_p);

    TestEeprmWriteBlock (wDeviceNo_p, pstrDaten_p, dwOffset_p, wSize);
}

//---------------------------------------------------------------------------
//
// Function:     TestEeprmSet
//
// Description:  Die Funktion schreibt Bytes in einen EEPROM.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestEeprmSet (char ** pArgList_p)
{
BYTE bDeviceNumber;

    // Device-No, Adresse und Daten definiert ?
    if (!pArgList_p[1] || !pArgList_p[2] || !pArgList_p[3])
    {
       printf("EEPROM write: Argument missing\r\n");
    }
    else
    {
       bDeviceNumber = (BYTE)htoul(pArgList_p[1]);

       // Startadresse (Offset) lesen
       dwEeprDumpAddr_l = htoul(pArgList_p[2]);

       // Daten in EEPROM schreiben
       TestEepromWrite (bDeviceNumber, dwEeprDumpAddr_l, pArgList_p[3]);

       printf("\r\n");
    }
}

#endif // TEST_EEPROM_SUPPORT

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test RTC>                                          */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Mit Hilfe der Klasse kann die Funktion der RTC berprft werden. Die
// Klasse untersttzt Funktionen zum Setzen der Uhrzeit/Datum, Lesen der
// aktuellen Uhrzeit/Datum.
//
/***************************************************************************/

#ifdef TEST_RTC_SUPPORT

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestRtcInit
//
// Description:  Die Funktion init. statische Variablen der Klasse.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestRtcInit (void)
{
    RtcInit();
}


//---------------------------------------------------------------------------
//
// Function:     TestRtcGetTimeDate
//
// Description:  Die Funktion ermittelt die aktuelle Zeit und Datum.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestRtcGetTimeDate (char ** pArgList_p)
{
tRealTime    RealTime;

    RtcGetAbsTime(&RealTime);

    printf("Time: ");
    printf("%02bu:", RealTime.bHour);
    printf("%02bu:", RealTime.bMinute);
    printf("%02bu ,", RealTime.bSecond);

    printf("Date: ");
    printf("%02u.", RealTime.bDay);
    printf("%02u.", RealTime.bMonth);
    printf("%04u\r\n", RealTime.wYears);

    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestRtcSetTimeDate
//
// Description:  Die Funktion setzt die aktuelle Zeit und Datum.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestRtcSetTimeDate (char ** pArgList_p)
{
tRealTime RealTime;
BYTE*     pDate;
BYTE*     pTime;

    if (!pArgList_p[1] || !pArgList_p[2])
    {
       printf("SRTC: Argument fehlt\r\n");
       return;
    }
    else
    {
        pTime = pArgList_p[1];
        pDate = pArgList_p[2];
        
        RealTime.bHour     = (pTime[0]-'0')*10   + (pTime[1]-'0');
        RealTime.bMinute   = (pTime[2]-'0')*10   + (pTime[3]-'0');
        RealTime.bSecond   = (pTime[4]-'0')*10   + (pTime[5]-'0');
        
        RealTime.bDay      = (pDate[0]-'0')*10   + (pDate[1]-'0');
        RealTime.bMonth    = (pDate[2]-'0')*10   + (pDate[3]-'0');
        RealTime.wYears    = (pDate[4]-'0')*1000 + (pDate[5]-'0')* 100
                         + (pDate[6]-'0')*10   + (pDate[7]-'0');
        
        RtcSetAbsTime (&RealTime);

        printf("\r\n");
    }
}

#endif // TEST_RTC_SUPPORT

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test Prozess Ein/Ausgnge>                         */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------
WORD bdata wPortValue;
WORD bdata wPortMask;

sbit wPortMask_0  = wPortMask^0;
sbit wPortMask_1  = wPortMask^1;
sbit wPortMask_2  = wPortMask^2;
sbit wPortMask_3  = wPortMask^3;

sbit wPortMask_8  = wPortMask^8;
sbit wPortMask_9  = wPortMask^9;
sbit wPortMask_10 = wPortMask^10;
sbit wPortMask_11 = wPortMask^11;
sbit wPortMask_12 = wPortMask^12;
sbit wPortMask_13 = wPortMask^13;
sbit wPortMask_14 = wPortMask^14;
sbit wPortMask_15 = wPortMask^15;

sbit wPortValue_0  = wPortValue^0;
sbit wPortValue_1  = wPortValue^1;
sbit wPortValue_2  = wPortValue^2;
sbit wPortValue_3  = wPortValue^3;

sbit wPortValue_8  = wPortValue^8;
sbit wPortValue_9  = wPortValue^9;
sbit wPortValue_10 = wPortValue^10;
sbit wPortValue_11 = wPortValue^11;
sbit wPortValue_12 = wPortValue^12;
sbit wPortValue_13 = wPortValue^13;
sbit wPortValue_14 = wPortValue^14;
sbit wPortValue_15 = wPortValue^15;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestAdGet
//
// Description:  Die Funktion gibt den aktuellen Zustand der analogen Eingnge aus. 
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestAdGet (char ** pArgList_p)
{
WORD i;

    printf("\r\n");
    // Anzeige Analoge-Eingnge
    printf ("\r\nAnaloge Eingnge  0 ...  4: ");
    for (i=0; i<4; i++)
        printf("%5.3fV ", (float)pImageInput_l->m_wAdc[i] * ONE_LSB);
    
    printf("\r\n");
}   

// -------------------------------------------------------------------------
//
// Function:     TestDiGet
//
// Description:  Die Funktion gibt die Werte der Prozesseingnge
//               ber RS-232 aus.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine
//
//
// State:        
//
// -------------------------------------------------------------------------

static void TestDiGet (char ** pArgList_p)
{
WORD i;

    printf("\r\n");

    //----------------------------------------------------------------------
    // Anzeige digitale Eingnge
    printf ("\r\nDigitale Eingnge 0 ... 31: %08lX", pImageInput_l->m_dwDIN);
    
    // Anzeige Zhler-Eingnge
    printf ("\r\nZhlereingnge    0 ...  3: ");
    for (i=0; i<4; i++)
        printf("%08lX ", pImageInput_l->m_liCnt[i]);
    
    printf("\r\n\n");
}

// -------------------------------------------------------------------------
//
// Function:     TestDoGet
//
// Description:  Die Funktion gibt die aktuellen Werte aus dem Prozessabbild der
//               Ausgnge aus. 
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine
//
//
// State:        
//
// -------------------------------------------------------------------------

static void TestDoGet (char ** pArgList_p)
{
    printf("\r\n");
    printf("\r\nDig. Ausgnge:   %08lX", pImageOutput_l->m_dwDOUT);
    printf("\r\nAna. Ausgnge:   %04X %04X", pImageOutput_l->m_wAout[0], 
                                           pImageOutput_l->m_wAout[1]);
    printf("\r\n\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestDoSet
//
// Description:  Die Funktion setzt einen neuen Ausgangswert im Prozessabbild.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestDoSet (char ** pArgList_p)
{
DWORD dwOut;

    // SDO-Kommando ohne Parameter?
    if (!strcmp(pArgList_p[0],"SDO"))
    {
       // Wert im Prozessabbild um ein Bit rotieren
       dwOut = pImageOutput_l->m_dwDOUT;
       dwOut = _lrol_(dwOut, 1);
       // Da in dem DWORD nur 20 Bit belegt sind, mu 
       // berlauf in Bit 20 auf Bit 0 bernommen werden.
       if (dwOut & 0x400000L)
          dwOut |= 0x00001L;
       dwOut &= 0x003FFFFF;
    }
    else
    {
       if (!pArgList_p[1])
       {
          printf("SDO: Argument fehlt\r\n");
          return;
       }

       //-----------------------------------------------------------------------
       dwOut  = htoul(pArgList_p[1]);
    }

    // Die Daten werden im Prozessabbild hinterlegt ..
    pImageOutput_l->m_dwDOUT = dwOut;

    printf("\r\n");
}

//---------------------------------------------------------------------------
//
// Function:     TestSetPort
//
// Description:  Die Funktion setzt ausgewhlte Port Pins.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestSetPort (char ** pArgList_p)
{
BYTE bPortNumber;

    if (!pArgList_p[1]  || !pArgList_p[2] || !pArgList_p[3])
    {
       printf("SP: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    bPortNumber  = (BYTE)htoul(pArgList_p[1]);
    wPortMask    = (WORD)htoul(pArgList_p[2]);
    wPortValue   = (WORD)htoul(pArgList_p[3]);

    switch(bPortNumber)
    {
        case 2:
                if (wPortMask_8)
                   P2_P8 = wPortValue_8;

                if (wPortMask_9)
                   P2_P9 = wPortValue_9;

                if (wPortMask_10)
                   P2_P10 = wPortValue_10;

                if (wPortMask_11)
                   P2_P11 = wPortValue_11;
        break;

        case 9:
                if (wPortMask_0)
                   P9_P0 = wPortValue_0;

                if (wPortMask_1)
                   P9_P1 = wPortValue_1;

                if (wPortMask_2)
                   P9_P2 = wPortValue_2;

                if (wPortMask_3)
                   P9_P3 = wPortValue_3;
        break;

        default: printf("\r\nDas gibts leider nicht.");
        break;
    }
}

//---------------------------------------------------------------------------
//
// Function:     TestGetPort
//
// Description:  Die Funktion liest ausgewhlte Port Pins.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestGetPort (char ** pArgList_p)
{
    printf("\r\n");
    printf("\r\nP2 = %04X", P2);
    printf("\r\nP3 = %04X", P3);
    printf("\r\nP4 = %04X", P4);
    printf("\r\nP5 = %04X", P5);
    printf("\r\nP7 = %04X", P7);

    printf("\r\n\n");
}


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test Prozess Ein/Ausgnge>                         */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/

#ifdef TEST_MODEM_CONTROL

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
#define SERIAL_INSTANCE 2

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

static CONST tSioCfgParam SioCfgParam_l =
{
    kSioDrvInitOk,      // Error Code
    57600L,             // Baudrate
    8,                  // Databits
    PARITY_NONE,        // Parity
    1,                  // Stopbits
    RTS_CTS_PROTOCOL    // Protocol
};

static tSerialInst FAR* pSioInst_l;

static tSioState SioState_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     TestModemInit
//
// Description:  Die Funktion init. die Modem-Instanz des SIO-Treibers.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestModemInit (void)
{
    // get pointer of the SioDrv instance
    pSioInst_l = SioGetInstData(SERIAL_INSTANCE);

    pSioInst_l->m_pfnSerialInit(&SioCfgParam_l);

    pSioInst_l->m_pfnGetState (&SioState_l);

}

//---------------------------------------------------------------------------
//
// Function:     TestRtsSet
//
// Description:  Die Funktion setzt den Status der RTS-Leitung.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestRtsSet           (char ** pArgList_p)
{
WORD wDaten;

    printf("\r\n");

    if (!pArgList_p[1])
    {
       printf("RTS: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    //-----------------------------------------------------------------------
    wDaten  = (WORD)htoul(pArgList_p[1]);

    if (wDaten)
       SioState_l.m_bRTS = SIGNAL_SET_ACTIVE;
    else 
       SioState_l.m_bRTS = SIGNAL_SET_INACTIVE;

    SioState_l.m_bDTR = SIGNAL_NOT_CHANGE;

    pSioInst_l->m_pfnSetState (&SioState_l);
}

//---------------------------------------------------------------------------
//
// Function:     TestDtrSet
//
// Description:  Die Funktion setzt den Status der DTR-Leitung.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestDtrSet (char ** pArgList_p)
{
WORD wDaten;

    printf("\r\n");

    if (!pArgList_p[1])
    {
       printf("RTS: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    wDaten  = (WORD)htoul(pArgList_p[1]);

    if (wDaten)
       SioState_l.m_bDTR = SIGNAL_SET_ACTIVE;
    else 
       SioState_l.m_bDTR = SIGNAL_SET_INACTIVE;

     SioState_l.m_bRTS = SIGNAL_NOT_CHANGE;

    pSioInst_l->m_pfnSetState (&SioState_l);
}

//---------------------------------------------------------------------------
//
// Function:     TestCtrGet
//
// Description:  Die Funktion liest den Status der Modem-Steuerleitung.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestCtrGet (char ** pArgList_p)
{
    printf("\r\n");

    //-----------------------------------------------------------------------
    pSioInst_l->m_pfnGetState (&SioState_l);

    printf("\r\nDTR = %d",SioState_l.m_bDTR?1:0);
    printf("\r\nRTS = %d",SioState_l.m_bRTS?1:0);
    printf("\r\nCTS = %d",SioState_l.m_bCTS?1:0);
    printf("\r\nDSR = %d",SioState_l.m_bDSR?1:0);
    printf("\r\nDCD = %d",SioState_l.m_bDCD?1:0);
    printf("\r\nRI  = %d",SioState_l.m_bRI?1:0);

    printf("\r\n");
}

#endif

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test PWM/PTO-FB>                                   */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

#define _FB_SUPPORT_PTO_PWM_

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define _LZS_DSALIGNMENT_1_
#define LZSTRUE                    TRUE
#define LZSFALSE                   FALSE
#define LZSBOOL                    BOOL
#define LZSWORD                    WORD
#define kIecOK                     0
#define kLzsSuccess                0
#define kIecGeneralError           0xFF

#define GETBIT(A)                  ((abDatensegment[bInstanz_l][(A)] == LZSTRUE)?TRUE:FALSE)
#define SETBIT(A,B)                (abDatensegment[bInstanz_l][(A)] = (B))
#define GETBYTE(A)                 (abDatensegment[bInstanz_l][(A)])
#define SETBYTE(A,B)               (abDatensegment[bInstanz_l][(A)] = (B))
#define GETWORD(A)                 (*(WORD *)&abDatensegment[bInstanz_l][(A)])
#define SETWORD(A,B)               (*(WORD *)&abDatensegment[bInstanz_l][(A)] = (B))
#define GETDWORD(A)                (*(DWORD *)&abDatensegment[bInstanz_l][(A)])
#define SETDWORD(A,B)              (*(DWORD *)&abDatensegment[bInstanz_l][(A)] = (B))

// Konstanten die in 1131-Header zu bernehmen sind
#define LZS_PTO_PWM_STATE           0x00
#define LZS_PTO_PWM_SET_TIMER       0x01
#define LZS_PTO_PWM_SET_PARAMETER   0x02
#define LZS_PTO_PWM_ENABLE          0x04
#define LZS_PTO_PWM_DISABLE         0x08
#define LZS_PTO_MODE                0x10
#define LZS_PWM_MODE                0x20

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

// Struktur mit deren Hilfe die Parameter an die Funktion ins
// LzsEnv.c bertragen werden.
typedef struct
{
    BYTE  m_bChannel;
    BYTE  m_bFbErrorCode;
    BYTE  m_bChannelState;
    BYTE  m_fTimeBase;
    WORD  m_wPeriode;
    WORD  m_wPulseDuration;
    int   m_iDelta;
    DWORD m_dwImpulse;

} tLzsPtoPwm;

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

static BYTE abPTODatensegment[2][0x20];
static BYTE bPTOInstanz_l;

#define abDatensegment abPTODatensegment
#define bInstanz_l     bPTOInstanz_l

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     IecRisingEdgeDetect
//
// Description:  Die Funktion ermittelt eine steigende Flanke am Eingang
//               ENABLE des FB's. Die Funktion ist aus dem LZS bernommen,
//               um den FB testen zu knnen.
//
//
// Parameters:   fSignal_p
//               wOffsCompSignal_p
//               
//
// 
// Returns:      Flanke
//
//
// State:        
//
//---------------------------------------------------------------------------

LZSBOOL  IecRisingEdgeDetect (LZSBOOL fSignal_p, LZSWORD wOffsCompSignal_p)
{

LZSBOOL  fRes;
   

   if (fSignal_p && !GETBIT(wOffsCompSignal_p))
   {
      fRes=LZSTRUE;
   }
   else
   {
      fRes=LZSFALSE;
   }
   SETBIT(wOffsCompSignal_p, fSignal_p);

   return (fRes);

}

#ifdef _FB_SUPPORT_PTO_PWM_

// Error Codes fr RTC-Funktionsbaustein
typedef enum
{
    kPtoPwmSuccessful       = 0,
    kPtoPwmInvalidMode      = 1,
    kPtoPwmHwError          = 2,
    kPtoPwmInvalidChannel   = 4,
    kPtoPwmBufferOverrun    = 8,
    kPtoPwmDeltaError       = 16

} tPtoPwmErrorCode;

//---------------------------------------------------------------------------
//
// Function:     LzsPtoPwm
//
// Description:  Die Funktion kapselt die hardwarespezifische Schnittstelle
//               fr den Funktionsbaustein. Der FB ist hardwareunabhngig
//               und ruft diese Funktion mit entsprechenden Kommandos auf, 
//               um die Verarbeitung zusteuern. 
//
//               Folgende Zustnde sind zu steuern:
//               - Konfiguration: Einstellen des Basis-Timers
//               - Parametrieren: Einstellen der Periodendauer, Impulsbreite,
//                                Impulsanzahl, Delta
//               - Starten des Generators
//               - Beenden des Generators
//
//               Die Kommandos werden als Bitmaske bergeben. Ein Funktions-
//               aufruf mu unter Umstnden mehrere Kommandos verarbeiten. Daher
//               ist die Reihenfolge der Anweisungen fr die richtige Ausfhrung
//               entscheidend (erst Parametrieren, dann Starten).
//
// Parameters:   bCommand_p: Kommando
//               pPtoPwm_p   Parameterliste fr das Kommando
//               
//
// 
// Returns:      errorcode
//
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC LzsPtoPwm(BYTE bCommand_p, tLzsPtoPwm * pPtoPwm_p)
{
BYTE bRetCode = C14_SUCCESSFUL;
BYTE bTimeBase;
BYTE bState = 0;

    if (bCommand_p & LZS_PTO_PWM_SET_TIMER)
    {
       bTimeBase = pPtoPwm_p->m_fTimeBase==LZSTRUE?PPW_TB_MILLI_SEC:PPW_TB_MICRO_SEC;

       // Basis-Timer init.
       bRetCode = C14PwmPtoTimer (pPtoPwm_p->m_bChannel, bTimeBase);
       if (bRetCode)
          goto EXIT;
    }

    //---------------------------------------------------------------------
    // PTO-Generator
    if (bCommand_p & LZS_PTO_MODE)
    {  
       // Neue Impulsbreite hinterlegen?
       // Ist der Generator nach Ablage der Parameter nicht mehr aktiv, 
       // so mu der Generator erneut gestartet werden. Die Enable-Funktion
       // prft, ob ein neuer Parametersatz vorliegt und startet den Generator
       // gegebenfalls.
       if (bCommand_p & LZS_PTO_PWM_SET_PARAMETER)
       {
          bRetCode = C14PtoParam(pPtoPwm_p->m_bChannel, pPtoPwm_p->m_wPeriode, pPtoPwm_p->m_iDelta, pPtoPwm_p->m_dwImpulse);
          if (bRetCode)
             goto EXIT;
       }

       // Generator starten ?
       if (bCommand_p & LZS_PTO_PWM_ENABLE)
       {
          // Generator nicht aktiv?
          bRetCode = C14PwmState(pPtoPwm_p->m_bChannel, &bState);
          if (bRetCode)
             goto EXIT;

          if (!(bState & CHN_BUSY))
          {
             // Generator starten
             bRetCode = C14PtoEnable(pPtoPwm_p->m_bChannel, TRUE);
             if (bRetCode)
                goto EXIT;
          }
       }

       // PWM-Generator ausschalten
       if (bCommand_p & LZS_PTO_PWM_DISABLE)
       {
          // Kanal ausschalten
          bRetCode = C14PtoEnable(pPtoPwm_p->m_bChannel, FALSE);
          goto EXIT;
       }
    }
    //---------------------------------------------------------------------
    // PWM-Generator
    else
    {
       // Neue Impulsbreite hinterlegen?
       if (bCommand_p & LZS_PTO_PWM_SET_PARAMETER)
       {
          bRetCode = C14PwmParam(pPtoPwm_p->m_bChannel, pPtoPwm_p->m_wPeriode, pPtoPwm_p->m_wPulseDuration);
          if (bRetCode)
             goto EXIT;
       }

       // PWM-Generator einschalten
       if (bCommand_p & LZS_PTO_PWM_ENABLE)
       {
          // Kanal einschalten
          bRetCode = C14PwmEnable(pPtoPwm_p->m_bChannel, TRUE);
          if (bRetCode)
             goto EXIT;
       }

       // PWM-Generator ausschalten
       if (bCommand_p & LZS_PTO_PWM_DISABLE)
       {
          // Kanal ausschalten
          bRetCode = C14PwmEnable(pPtoPwm_p->m_bChannel, FALSE);
          goto EXIT;
       }
    }

EXIT:
    //-------------------------------------------------------------
    // Errorcode umcodieren nach LZS-Code
    switch (bRetCode)
    {
        case C14_SUCCESSFUL:        bRetCode = kPtoPwmSuccessful;
        break;

        case C14_BUFFER_OVERRUN:    bRetCode = kPtoPwmBufferOverrun;
        break;

        case C14_INVALID_CHANNEL:   bRetCode = kPtoPwmInvalidChannel;
        break;

        default: bRetCode = kPtoPwmInvalidMode;
        break;
    }


    //--------------------------------------------------------------    
    // aktuellen Ausfhrungszustand ermitteln
    bState = 0;
    C14PwmState(pPtoPwm_p->m_bChannel, &bState);

    // Bisher noch kein Fehler, dann Fehlerflags prfen
    if (!bRetCode)
    {
       if (bState & CHN_DELTA_ERROR)
          bRetCode = kPtoPwmDeltaError;
    }

    //--------------------------------------------------------------    
    // aktuellen Ausfhrungszustand ablegen
    pPtoPwm_p->m_bChannelState = (bState & CHN_BUSY)?LZSTRUE:LZSFALSE;
    pPtoPwm_p->m_bFbErrorCode  = bRetCode;

    return (kLzsSuccess);
}

//---------------------------------------------------------------------------
// Extended-Funktionsbaustein:  SPTO_PWM (ENABLE, MODE, CHANNEL, APPEND
//                                        TB, CT, PT, DELTA, PC |
//                                        READY, BUSY, ERROR)
//---------------------------------------------------------------------------
// 
// 
//                 +--------------------+
//                 |      SPTO_PWM      |
//                 |                    |
//       BOOL   ---|ENABLE         READY|--- BOOL
//                 |                BUSY|--- BOOL
//       BOOL   ---|MODE                |
//       USINT  ---|CHANNEL             |
//       BOOL   ---|APPEND              |
//                 |                    |
//       BOOL   ---|TB                  |
//       UINT   ---|CT                  |
//       UINT   ---|PT                  |
//       INT    ---|DELTA               |
//       UDINT  ---|PC                  |
//                 |               ERROR|--- USINT
//                 |                    |
//                 +--------------------+
// 
// 
// Input:
// ======
// 
// ENABLE     (BOOL)   - Freigabe PTO/PWM-Generators
//                       TRUE  : Generator einschalten, Steuerung des Ausgang
//                               durch den Generator
//                       FALSE : Generator ausschalten, Steuerung des Ausgangs 
//                               durch Prozessabbild, Ausgnge des FB zurcksetzen
// 
//                       Mit der steigenden Flanke am Eingang bernimmt der Baustein
//                       den Wert fr den Basistakt (Eingang TB).
// 
// MODE       (BOOL)   - Auswahl Betriebsart
//                       TRUE  : PTO-Generator (Impulszhlerausgang)
//                       FALSE : PWM-Generator (Impulsdauerausgang)
// 
//                       Eine nderung des Mode-Eingangs bei gesetztem ENABLE-Eingang 
//                       fhrt zum Beenden der vorher eingestellten Funktion.
// 
// CHANNEL    (USINT)  - Auswahl Kanal
//                       0     : Kanal P0
//                       1     : Kanal P1
// 
// APPEND     (BOOL)   - Parametersatz anfgen
//                       TRUE  : Ein weiterer Parametersatz ist zu bernehmen.
//                       FALSE : Es werden keine Parameter bernommen. Der 
//                               FB aktualisiert seine Status-Ausgnge. 
// 
//                       Fr das ndern von Parametern des PWM-Generators oder das Anfgen 
//                       eines Parametersatzes beim PTO-Generator ist der Eingang auf TRUE 
//                       zu setzen.
// 
// TB         (BOOL)   - Auswahl des Basistaktes fr den PTO/PWM-Generator
//                       TRUE  : 1ms Basistakt
//                       FALSE : 800ns Basistakt
// 
// CT         (UINT)   - Zykluszeit, Periodendauer
//                      TB = TRUE:  2 ... 65535 (2ms - 65535ms)
//                       TB = FALSE: 125 ... 65535 (100s - 52428 s)
// 
// PT         (UINT)   - PWM-Generator: Impulsdauer
//                       PTO-Generator: keine Verwendung
//                       Wertebereich: 0 .. 65535
// 
// DELTA      (INT)    - PTO-Generator: nderung der Periodendauer zwischen zwei Impulsen
//                       PWM-Generator: keine Verwendung
//                       Wertebereich: -32768 .. +32767
// 
// PC         (UDINT)  - PTO-Generator: Anzahl Impulse
//                       PWM-Generator: keine Verwendung
//                       Wertebereich: 1 .. 4294967295
// 
// Bei Unterschreiten der Bereichsgrenze wird der Eingang auf den untersten Wert gesetzt.
// 
// Output:
// =======
// 
// READY      (BOOL)   - Status-Ausgang
//                       TRUE  : FB wurde vollstndig parametriert, Generator bereit.
//                       FALSE : FB wurde mit einem Fehler beendet, der Generator
//                               ist nicht bereit.
// 
// BUSY       (BOOL)   - Status-Ausgang
//                       TRUE  : Die Steuerung des digitalen Ausgangs erfolgt durch
//                               den Generator. 
//                       FALSE : Der Generator ist nicht aktiv, die Steuerung des
//                               digitalen Ausgangs erfolgt durch das Prozessabbild.
// 
// ERROR      (USINT)  - Error-Ausgang
//                       0     : ok
//                       1     : Es wurde versucht, den FB in einer ungltigen 
//                               Betriebsart zu verwenden. 
//                       2     : Ein Fehler in der Hardware wurde erkannt.
//                       4     : Es wurde versucht, einen ungltigen Kanal zu verwenden.
//                       8     : Die interne Tabelle fr die Ablage eines Parametersatzes 
//                               ist voll. Es knnen keine weiteren Parameter aufgenommen
//                               werden. 
//                      16     : Bei der Berechnung der Periodendauer wurden die Bereichs-
//                               grenzen des Wertebereichs berschritten. 
//                               TB = TRUE:  2 ... 65535 (2ms - 65535ms)
//                               TB = FALSE: 125 ... 65535 (100s - 52428 s)
// 
// 
// Beschreibung:
// Der Baustein definiert alternativ zur Standardfunktion der Digitalen Ausgnge P0 und P1
// die Ausgnge als Impulsfolgeausgang (PTO) oder Impulsdauerausgang (PWM). Ist der
// Eingang ENABLE = FALSE, so wird der digitale Ausgang ber das Prozessabbild 
// gesetzt. Ist der Eingang ENABLE = TRUE, so wird der Ausgang durch den PTO/PWM-
// Generator gesetzt. 

// PWM-Generator (MODE = FALSE):
// In der Funktion als PWM-Generator wird eine kontinuierliche Impulsfolge am 
// digitalen Ausgang erzeugt. Die Periodendauer als auch die Impulsdauer sind 
// als Anzahl der Basistakte einstellbar. Der PWM-Generator wird unmittelbar 
// nach der bergabe der Parameter eingeschaltet. Das Erzeugen des Signals 
// wird beendet, wenn der Baustein mit dem Eingang ENABLE = FALSE aufgerufen wird,
// die Zeitbasis sich ndert oder die Betriebsart umgeschaltet wird.
// Wird der Wert 0 als Impulsdauer PT bergeben, so geht der Ausgang fr die gesamte
// Periodendauer in einen inaktiven Zustand. Wird fr die Impulsdauer PT ein Wert 
// grer oder gleich der Periodendauer bergeben, so geht der Ausgang fr die 
// gesamte Periodendauer in den aktiven Zustand.
// Das ndern der Periodendauer erfolgt stets asynchron. Die laufende Periode wird 
// unterbrochen, um den neuen Wert zu bernehmen. 
// Das ndern der Impulsdauer erfolgt synchron. Die Impulsdauer wird beim Starten der
// nchsten Periodendauer bernommen. 
// Fr das ndern von Parametern ist der Eingang APPEND auf TRUE zusetzen.
// 
// PTO-Generator (MODE = TRUE):
// Der PTO-Generator erzeugt eine Impulsfolge und steuert damit den Digitalen Ausgang.
// Eine Impulsfolge wird durch einen Parametersatz beschrieben. Die Impulsfolge ist
// parametrierbar hinsichtlich der Periodendauer (Anfangswert), einem Delta der 
// Periodendauer und der Anzahl von Impulsen. Die Impulsbreite wird fest auf 50% der 
// Periodendauer eingestellt. Ist eine Impulsfolge vollstndig bertragen und ist kein
// weiterer Parametersatz vorhanden, so schaltet der PTO-Generator ab und die Steuerung 
// des digitalen Ausgangs erfolgt wieder durch das Prozessabbild. Im Prozessabbild ist daher 
// der gewnschte Zustand fr den digitalen Ausgang zu hinterlegen, falls der PTO-
// Generator nicht aktiv ist.
// Der FB untersttzt das Aneinanderreihen von Impulsfolgen unterschiedlicher Werte
// fr die Periodendauer, Periodendauer-Delta und Anzahl der Impulse. Es knnen bis 
// zu 255 Parameterstze aneinandergehngt werden. Werden mehr eingetragen, so meldet 
// der FB den Fehler 8 (OVERRUN). Fr das Anreihen von Parameterstzen ist der Eingang
// APPEND auf TRUE zu setzen.
// 
// Tritt ein Fehler auf, so schaltet der FB ab. Eine erneute Verwendung des FB ist
// nur dann mglich, wenn der FB mit dem Eingang ENABLE = FALSE aufgerufen wurde. 
// 
// Der Ausgang BUSY signalisiert, da eine parametrierte Impulsfolge bertragen wird
// (PTO-Funktion). In der Betriebsart PWM ist der Ausgang stets auf TRUE, solange der
// Eingang ENABLE = TRUE.
// 
// Der Ausgang READY signalisiert, dass der Baustein vollstndig parametriert ist.
// Eine nderung des Parameters Basis-Timer TB ist nicht mehr mglich (nur bei 
// steigender Flanke am Eingang ENABLE). Der READY-Ausgang geht nach FALSE, wenn 
// der Baustein mit ENABLE = FALSE aufgerufen wird.
// 
// Festlegungen fr Offsets der Parameter im Datensegment:
// Die Ausrichtung erfolgt unter Bercksichtigung des Alignment.
// _LZS_DSALIGNMENT_1_ -> BYTE, WORD, DWORD sind auf BYTE-Adressen abzubilden
// _LZS_DSALIGNMENT_2_ -> WORD, DWORD sind auf WORD-Adressen abzubilden
// _LZS_DSALIGNMENT_4_ -> DWORD sind auf DWORD-Adressen abzubilden


#ifdef  _LZS_DSALIGNMENT_1_
    #define     SPPW_ENABLE         0x09    // BOOL:  Size=1
    #define     SPPW_MODE           0x0A    // BOOL:  Size=1
    #define     SPPW_CHANNEL        0x0B    // USINT: Size=1
    #define     SPPW_APPEND         0x0C    // BOOL:  Size=1
    #define     SPPW_TB             0x0D    // BOOL:  Size=1
    #define     SPPW_CT             0x0E    // UINT:  Size=2
    #define     SPPW_PT             0x10    // UINT:  Size=2
    #define     SPPW_DELTA          0x12    // INT:   Size=2
    #define     SPPW_PC             0x14    // UDINT: Size=4

    #define     SPPW_READY          0x06    // BOOL:  Size=1
    #define     SPPW_BUSY           0x07    // BOOL:  Size=1
    #define     SPPW_ERROR          0x08    // USINT: Size=1

    #define     SPPW_M_ENABLE       0x18    // Merker-Bit fr Flankenerk.
    #define     SPPW_M_MODE         0x19    // Merker-Bit fr Flankenerk.
#endif

#ifdef  _LZS_DSALIGNMENT_2_
    #define     SPPW_ENABLE         0x09    // BOOL:  Size=1
    #define     SPPW_MODE           0x0A    // BOOL:  Size=1
    #define     SPPW_CHANNEL        0x0B    // USINT: Size=1
    #define     SPPW_APPEND         0x0C    // BOOL:  Size=1
    #define     SPPW_TB             0x0D    // BOOL:  Size=1
    #define     SPPW_CT             0x0E    // UINT:  Size=2
    #define     SPPW_PT             0x10    // UINT:  Size=2
    #define     SPPW_DELTA          0x12    // INT:   Size=2
    #define     SPPW_PC             0x14    // UDINT: Size=4

    #define     SPPW_READY          0x06    // BOOL:  Size=1
    #define     SPPW_BUSY           0x07    // BOOL:  Size=1
    #define     SPPW_ERROR          0x08    // USINT: Size=1

    #define     SPPW_M_ENABLE       0x18    // Merker-Bit fr Flankenerk.
#endif

#ifdef  _LZS_DSALIGNMENT_4_
    #define     SPPW_ENABLE         0x09    // BOOL:  Size=1
    #define     SPPW_MODE           0x0A    // BOOL:  Size=1
    #define     SPPW_CHANNEL        0x0B    // USINT: Size=1
    #define     SPPW_APPEND         0x0C    // BOOL:  Size=1
    #define     SPPW_TB             0x0D    // BOOL:  Size=1
    #define     SPPW_CT             0x0E    // UINT:  Size=2
    #define     SPPW_PT             0x10    // UINT:  Size=2
    #define     SPPW_DELTA          0x12    // INT:   Size=2
    #define     SPPW_PC             0x14    // UDINT: Size=4

    #define     SPPW_READY          0x06    // BOOL:  Size=1
    #define     SPPW_BUSY           0x07    // BOOL:  Size=1
    #define     SPPW_ERROR          0x08    // USINT: Size=1

    #define     SPPW_M_ENABLE       0x18    // Merker-Bit fr Flankenerk.
#endif

//
//---------------------------------------------------------------------------

BYTE ExtFB_SPtoPwm (void)
{

BYTE  bRetCode;
BYTE  bRes;
BYTE  bCommand;
BOOL  fBusy;
BOOL  fAppend;
tLzsPtoPwm PtoPwm;

    bRes     = kIecOK;
    fBusy    = LZSFALSE;
    bCommand = 0;

    // Auswahl Kanalnummer 0 - 255: Der Wertebereich fr die 
    // Kanalnummern wird durch den Treiber definiert. Daher 
    // erfolgt hier keine berprfung des Wertebereichs. 
    PtoPwm.m_bChannel     = GETBYTE(SPPW_CHANNEL);
    PtoPwm.m_bFbErrorCode = C14_SUCCESSFUL;

    //--------------------------------------------------------------------------
    // ENABLE = TRUE: --> Steuerung durch PTO/PWM-Generator starten
    if ( GETBIT(SPPW_ENABLE) )
    {
        bCommand  = (GETBIT(SPPW_MODE))?LZS_PTO_MODE:LZS_PWM_MODE;
        fAppend   = GETBIT(SPPW_APPEND);

        //----------------------------------------------------------------
        // Wurde der Baustein das erste Mal mit ENABLE = TRUE gerufen,
        // so sind die Werte fr den Basis-Timer und, im Falle von PWM, 
        // die Periodendauer zu bernehmen. 
        // ndert sich der MODE-Eingang nachdem eine Funktion
        // enabled wurde, so wird die bisherige Funktion beendet
        // und die neue Funktion init. 
        if ( IecRisingEdgeDetect(GETBIT(SPPW_ENABLE), SPPW_M_ENABLE) ||
             (GETBIT(SPPW_MODE) != GETBIT(SPPW_M_MODE) ) )
        {
           // Wurde das SPS-Programm gestoppt, whrend der Generator 
           // aktiv war, so ist die HW noch init. (Interrupts aktiv).
           // In diesem Fall ist beim erneuten Starten des SPS-Programms
           // der Generator zu stoppen.
           bRetCode = LzsPtoPwm(bCommand | LZS_PTO_PWM_DISABLE, &PtoPwm);
           if (bRetCode != kLzsSuccess)
           {
              bRes = kIecGeneralError;
              goto EXIT;
           }
           
           bCommand |= (LZS_PTO_PWM_SET_TIMER | LZS_PTO_PWM_ENABLE);
           fAppend   = TRUE;                       // bernahme Parameter

           PtoPwm.m_fTimeBase = GETBIT(SPPW_TB);   // Basis-Timer

           SETBIT(SPPW_READY, LZSTRUE);
           SETBIT(SPPW_M_MODE, GETBIT(SPPW_MODE));
        }

        //----------------------------------------------------------------
        // Nach einem Fehler wird das CONFIRM-Bit gelscht. Die 
        // weitere Ausfhrung kann erst dann wieder erfolgen, wenn
        // der Baustein mit ENABLE = FALSE aufgerufen wurde. Dadurch 
        // werden smtliche Ausgnge zurckgesetzt.
        if ( !GETBIT(SPPW_READY) )
        {
            // Ausgangswerte unverndert belassen
            goto EXIT;
        }

        //-----------------------------------------------------------
        // Neuen Parametersatz hinterlegen/anhngen
        //-----------------------------------------------------------
        if (fAppend)
        {
           // Parameter bernehmen
           bCommand |= LZS_PTO_PWM_SET_PARAMETER;

           PtoPwm.m_wPeriode  = GETWORD(SPPW_CT);      // Periodendauer
           // Bei gesetztem APPEND-Eingang wird ein weiterer
           // Parametersatz an den PTO/PWM-Generator bergeben.
           // PTO: Periodendauer, Delta, Impulsanzahl
           // PWM: Impulsbreite
           // PTO-Ausgang parametrieren?
           if (GETBIT(SPPW_MODE))
           {
              PtoPwm.m_iDelta    = GETWORD(SPPW_DELTA);   // Delta Periodendauer
              PtoPwm.m_dwImpulse = GETDWORD(SPPW_PC);
              // Voreinstellung Zhlwert fr Impulse ist 1.
              if (PtoPwm.m_dwImpulse == 0L)
                 PtoPwm.m_dwImpulse = 1L;

              bCommand |= LZS_PTO_PWM_ENABLE;
           }
           // PWM-Ausgang parametrieren
           else
           {
              // PWM-Generator: Impulsdauer lesen
              PtoPwm.m_wPulseDuration = GETWORD(SPPW_PT);
           }
        }

        //-----------------------------------------------------------
        // PTO/PWM-Generator parametrieren, Ausfhrungsstatus abfragen ...
        //-----------------------------------------------------------
        // Der Funktionsbaustein besitzt einen BUSY-Ausgang und Errorausgang.
        // ber den BUSY-Ausgang wird angezeigt, ob die parametrierte 
        // Impulsfolge vollstndig bertragen wurde. 
        // PTO: BUSY = TRUE  ---> Generator arbeitet
        //      BUSY = FALSE ---> Generator im Leerlauf
        // PWM: BUSY = TRUE  ---> Generator luft kont.
        bRetCode = LzsPtoPwm(bCommand, &PtoPwm);
        if (bRetCode != kLzsSuccess)
        {
           bRes = kIecGeneralError;
           goto EXIT;
        }

        // ErrorCode (z.B. berlauf) am FB-Ausgang setzen
        SETBYTE(SPPW_ERROR, PtoPwm.m_bFbErrorCode);

        // Bei einem Fehler wird der CONFIRM-Ausgang auf FALSE
        // gesetzt. Der Baustein ist durch den Anwender mit
        // ENABLE = FALSE aufzurufen.
        if (PtoPwm.m_bFbErrorCode)
        {
           SETBIT(SPPW_READY, LZSFALSE);
           // Kanal nach Fehler sperren (lscht interne Fehlerflags)
           LzsPtoPwm(bCommand | LZS_PTO_PWM_DISABLE, &PtoPwm);
        }

        // aktuellen Ausfhrungszustand hinterlegen
        fBusy = (PtoPwm.m_bChannelState == TRUE)?LZSTRUE:LZSFALSE;

        // BUSY-Ausgang setzen
        SETBIT(SPPW_BUSY, fBusy);
    }
    //--------------------------------------------------------------------------
    // ENABLE = FALSE: --> Steuerung durch PTO/PWM-Generator sperren
    else
    {
        bCommand  = (GETBIT(SPPW_MODE))?LZS_PTO_MODE:LZS_PWM_MODE;
        // Kanal sperren
        bRetCode = LzsPtoPwm(bCommand | LZS_PTO_PWM_DISABLE, &PtoPwm);
        if (bRetCode != kLzsSuccess)
        {
           bRes = kIecGeneralError;
           goto EXIT;
        }

        // ErrorCode (z.B. ungltige Kanalnummer) am FB-Ausgang setzen
        SETBYTE(SPPW_ERROR, PtoPwm.m_bFbErrorCode);
        
        // Ausfhrung beendet
        SETBIT (SPPW_BUSY, LZSFALSE);

        // Ausfhrung beendet
        SETBIT (SPPW_READY, LZSFALSE);

        // Merker zurcksetzen
        SETBIT (SPPW_M_ENABLE, LZSFALSE);
        SETBIT (SPPW_M_MODE,   LZSFALSE);
    }


EXIT:

    return (bRes);
}

#endif

//---------------------------------------------------------------------------
//
// Function:     TestPwmInit
//
// Description:  Die Funktion init. das Datensegment des FB's.
//
//
// Parameters:   void
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestPwmInit (void)
{
    bInstanz_l = 0;
    SETBIT(SPPW_ENABLE,   LZSFALSE);    
    SETBYTE(SPPW_CHANNEL, 0);
    SETBIT(SPPW_MODE,     LZSFALSE);
    SETBIT(SPPW_APPEND,   LZSFALSE);
    SETBIT(SPPW_TB,       LZSFALSE);
    SETWORD(SPPW_CT,      0);
    SETWORD(SPPW_PT,      0);
    SETWORD(SPPW_DELTA,   0);
    SETDWORD(SPPW_PC,     0L);
    SETBIT(SPPW_READY,    LZSFALSE);
    SETBIT(SPPW_BUSY,     LZSFALSE);
    SETBYTE(SPPW_ERROR,   0);
    SETBYTE(SPPW_M_ENABLE,0);

    bInstanz_l = 1;
    SETBIT(SPPW_ENABLE,   LZSFALSE);    
    SETBYTE(SPPW_CHANNEL, 0);
    SETBIT(SPPW_MODE,     LZSFALSE);
    SETBIT(SPPW_APPEND,   LZSFALSE);
    SETBIT(SPPW_TB,       LZSFALSE);
    SETWORD(SPPW_CT,      0);
    SETWORD(SPPW_PT,      0);
    SETWORD(SPPW_DELTA,   0);
    SETDWORD(SPPW_PC,     0L);
    SETBIT(SPPW_READY,    LZSFALSE);
    SETBIT(SPPW_BUSY,     LZSFALSE);
    SETBYTE(SPPW_ERROR,   0);
    SETBYTE(SPPW_M_ENABLE,0);

    bInstanz_l = 0;
}

//---------------------------------------------------------------------------
//
// Function:     TestPwmSet
//
// Description:  Die Funktion schreibt die PWM-Ausgnge. Fr den Zugriff 
//               wird der FB SPTO_PWM verwendet. Die Parameter werden 
//               im Datensegment des Bausteins hinterlegt.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestPwmSet (char ** pArgList_p)
{
BYTE bPortNumber;
WORD wPulse;
WORD wPeriode;

    printf("\r\n");

    if (!pArgList_p[1]  || !pArgList_p[2] || !pArgList_p[3])
    {
       printf("SPWM: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    bPortNumber  = (BYTE)htoul(pArgList_p[1]);
    wPeriode     = (WORD)htoul(pArgList_p[2]);
    wPulse       = (WORD)htoul(pArgList_p[3]);

    if (bPortNumber > 3)
    {
       printf("SPWM: Invalid Port Number (0 - 3)\r\n");
       return;
    }

    if (bPortNumber < 2)
    {
       bInstanz_l = bPortNumber;

       // Aktueller Zustand der Ausgnge vor dem Aufruf des Bausteins
       // anzeigen:
       printf("\r\nCONFIRM %d, BUSY %d, ERROR %02bX", 
                   GETBIT(SPPW_READY)?1:0, GETBIT(SPPW_BUSY)?1:0, GETBYTE(SPPW_ERROR));

       // Ist die parametrierte Periodendauer ungleich Null
       // so wird der Kanal enabled, anderenfalls disabled.
       // Impulsdauer auf Anfangswert: 0% Einschaltdauer
       if (!wPeriode)
       {  
          // Eingnge des FB setzen --> Datensegment schreiben
          SETBYTE(SPPW_ENABLE,  LZSFALSE);     // PWM-Generator ausschalten
          SETBYTE(SPPW_CHANNEL, bPortNumber);  // Kanalnummer
          ExtFB_SPtoPwm();
       }
       else
       {
          SETBIT(SPPW_ENABLE,   LZSTRUE);      // PWM-Generator einschalten
          SETBYTE(SPPW_CHANNEL, bPortNumber);  // Kanalnummer
          SETBIT(SPPW_MODE,     LZSFALSE);
          SETBIT(SPPW_APPEND,   LZSTRUE);
          SETWORD(SPPW_CT,      wPeriode);
          SETWORD(SPPW_PT,      wPulse);
          ExtFB_SPtoPwm();
       }

       // Aktueller Zustand der Ausgnge nach dem Aufruf des Bausteins
       // anzeigen:
       printf("\r\nCONFIRM %d, BUSY %d, ERROR %02bX", 
                   GETBIT(SPPW_READY)?1:0, GETBIT(SPPW_BUSY)?1:0, GETBYTE(SPPW_ERROR));
    }
    else
    {
       pImageOutput_l->m_wAout[bPortNumber-2] = wPulse & 0x7FFF;

       C14AoutSet((BYTE FAR*)&ImageOutput_l.m_wAout[0], sizeof(ImageOutput_l.m_wAout));
    
       printf("\r\nREL = %04X, PWM2=%04X, PWM3=%04X", CC2_T8REL, CC2_CC18, CC2_CC19);
    }
}

//---------------------------------------------------------------------------
//
// Function:     TestPtoSet
//
// Description:  Die Funktion schreibt eine Impulsfolge.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestPtoSet (char ** pArgList_p)
{
BYTE  bPortNumber;
WORD  wDelta;
WORD  wPeriode;
DWORD dwImpulse;

    printf("\r\n");

    if (!pArgList_p[1]  || !pArgList_p[2] || !pArgList_p[3] || !pArgList_p[4])
    {
       printf("SPTO: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    bPortNumber  = (BYTE)htoul(pArgList_p[1]);
    wPeriode     = (WORD)htoul(pArgList_p[2]);
    wDelta       = (WORD)htoul(pArgList_p[3]);
    dwImpulse    = (DWORD)htoul(pArgList_p[4]);

    if (bPortNumber > 2)
    {
       printf("SPTO: Invalid Port Number (0 - 1)\r\n");
       return;
    }

    bInstanz_l = bPortNumber;

    // Aktueller Zustand der Ausgnge vor dem Aufruf des Bausteins
    // anzeigen:
    printf("\r\nCONFIRM %d, BUSY %d, ERROR %02bX", 
                GETBIT(SPPW_READY)?1:0, GETBIT(SPPW_BUSY)?1:0, GETBYTE(SPPW_ERROR));

    // Ist die parametrierte Periodendauer ungleich Null
    // so wird der Kanal enabled, anderenfalls disabled.
    // Impulsdauer auf Anfangswert: 0% Einschaltdauer
    if (!wPeriode)
    {  
       // Eingnge des FB setzen --> Datensegment schreiben
       SETBYTE(SPPW_ENABLE,  LZSFALSE);     // PWM-Generator ausschalten
       SETBYTE(SPPW_CHANNEL, bPortNumber);  // Kanalnummer
       ExtFB_SPtoPwm();
    }
    else
    {
       SETBIT(SPPW_ENABLE,   LZSTRUE);      // PWM-Generator einschalten
       SETBYTE(SPPW_CHANNEL, bPortNumber);  // Kanalnummer
       SETBIT(SPPW_MODE,     LZSTRUE);      // PTO-Generator
       SETBIT(SPPW_APPEND,   LZSTRUE);
       SETWORD(SPPW_CT,      wPeriode);
       SETWORD(SPPW_DELTA,   wDelta);
       SETDWORD(SPPW_PC,     dwImpulse);
       ExtFB_SPtoPwm();
    }
/*
    // Warten bis Impulsfolge bertragen
    printf("\r\nWaiting for sending pulses ... ");
    SETBIT(SPPW_APPEND,   LZSFALSE);  // nur Status lesen
    do 
    {
       ExtFB_SPtoPwm();

    } while(GETBIT(SPPW_BUSY));

    printf(" ok");
*/
    // Aktueller Zustand der Ausgnge nach dem Aufruf des Bausteins
    // anzeigen:
    printf("\r\nCONFIRM %d, BUSY %d, ERROR %02bX", 
                GETBIT(SPPW_READY)?1:0, GETBIT(SPPW_BUSY)?1:0, GETBYTE(SPPW_ERROR));
}

//---------------------------------------------------------------------------
//
// Function:     TestPtoPwmTbSet
//
// Description:  Die Funktion definiert den Basistimer fr einen PTO/PWM-Kanal.
//
//
// Parameters:   Pointer auf Argumentenliste
//
// 
// Returns:      keine 
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TestPtoPwmTbSet (char ** pArgList_p)
{
BYTE  bPortNumber;
BYTE  bTimeBase;

    printf("\r\n");

    if (!pArgList_p[1]  || !pArgList_p[2])
    {
       printf("STB: Argument fehlt\r\n");
       return;
    }

    //-----------------------------------------------------------------------
    bPortNumber  = (BYTE)htoul(pArgList_p[1]);
    bTimeBase    = (BYTE)htoul(pArgList_p[2]);

    if (bPortNumber > 2)
    {
       printf("SPTO: Invalid Port Number (0 - 1)\r\n");
       return;
    }

    bInstanz_l = bPortNumber;

    // Generator ausschalten und neue Zeitbasis im Datensegment hinterlegen
    SETBYTE(SPPW_ENABLE,  LZSFALSE);     // PWM-Generator ausschalten
    SETBYTE(SPPW_CHANNEL, bPortNumber);  // Kanalnummer
    SETBIT(SPPW_TB, (bTimeBase==0)?LZSFALSE:LZSTRUE);
    ExtFB_SPtoPwm();
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Test RTOS>                                         */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static void TestHandlerAdce (void);
static void TestHandlerPll  (void);
static void TestHandlerCOP  (void);
static void TestHandlerWdt  (void);


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     TestRtosInit
//
// Description:  Die Funktion initialisiert die Tasks.
//
//
// Parameters:   none
//               
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC TestRtosInit(void)
{
    RtosInit();

    RtosInitWatchdog(TestHandlerWdt); 

    RtosCreateTask(TASK_ADC_ERROR, TestHandlerAdce);
    RtosCreateTask(TASK_TMR_COM,   TestHandlerCOP);
    RtosCreateTask(TASK_PLL_ERROR, TestHandlerPll);

    RtosDelayTask(TASK_TMR_COM,   1000);
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

static void TestHandlerAdce (void)
{ 
    // tritt nie ein, da AD-Wandler wartet, bis letzter Wert abgeholt
    // wurde bevor neue Wandlung startet
    printf("\nFatal Error: ADC overrun --> machine stoped");
    while(1);
}

static void TestHandlerPll (void)
{
    printf("\nFatal Error: PLL error --> machine stoped");
    while(1);
}

static void TestHandlerCOP (void)
{
    P3 ^= 0x0010;
    RtosDelayTask(TASK_TMR_COM,   300000);
}

static void TestHandlerWdt (void)
{
    P3 &= 0xFFF7;
    bWatchdogEvent_l = TRUE;
}

void PUBLIC TestPowerFailHandler(bit bitPowerFailState)
{
    // Wenn Power Fail whrend Critical Section, dann 
    // sofort Ausgnge sperren und Peripherie stoppen
    if (bitPowerFailState)
    {   // Ausgnge disablen
        C14ShutDown ();

        // weitere Interrupts sperren (CAN, Ethernet, ...)
        // ...
    }
}

