/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  include file for eeprom driver api function

  -------------------------------------------------------------------------

                $RCSfile: EeprmDrv.h,v $
                
                $Author: A.Suess $
                
                $Revision: 1.3 $  $Date: 2005/01/04 12:12:54 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  1999/01/08 -rs:   start of implemtation
  2001/12/10 -ct:   adjust for the DIPmodul 164
  2003/10/28 -ct:   adjust for the XC161
  2003/06/07 -rs:   V2.10 (additional sector for PLC configuration included)

****************************************************************************/

#ifndef _EEPRMDRV_H_
#define _EEPRMDRV_H_



//---------------------------------------------------------------------------
//  Konfiguration
//---------------------------------------------------------------------------

#define _SUPPORT_PLC_CFG_SECT_          // Erweiterung zum Sichern von
                                        // Konfigurations-Daten im EEPROM
                                        // aktivieren/deaktivieren



//---------------------------------------------------------------------------
//  Versionsnummer
//---------------------------------------------------------------------------

#define DOMAIN      2                   // Versionsnummer 2.10
#define REVISON     1



//---------------------------------------------------------------------------
//  Kostanten
//---------------------------------------------------------------------------

// Konstanten zur Typauswahl
#define EPRM25080   0x01                //  1 kByte
#define EPRM25160   0x02                //  2 kByte
#define EPRM25170   0x03                //  2 kByte
#define EPRM25640   0x04                //  8 kByte
#define EPRM25650   0x05                //  8 kByte
#define EPRM25128   0x06                // 16 kByte
#define EPRM25256   0x07                // 32 kByte
#define EPRM25512   0x08                // 64 kByte

#define TWO_EEPROM  0x80                // Flag fuer 2 bestckte EEPROM's

// Fehlercodes
#define EEPROM_SUCCESSFUL   0x00
#define EEPROM_UNKNOWN_TYPE 0xFF



//---------------------------------------------------------------------------
//  Typen
//---------------------------------------------------------------------------

typedef struct
{
    BYTE    m_bDeviceType;
    DWORD   m_dwCfgSectSize;
    DWORD   m_dwPdatSectSize;

} tEprmInitCfg;


typedef struct
{
    DWORD   m_dwCfgSectSize;
    DWORD   m_dwProgSectSize;
    DWORD   m_dwPdatSectSize;

} tEprmSectSize;



//---------------------------------------------------------------------------
//  Prototypen
//---------------------------------------------------------------------------
// initialize the eeprom driver
WORD  EprmInit (tEprmInitCfg FAR* pEprmInitCfg_p);

// get the data size for the usable eeprom area
BYTE  EprmGetSectSize (tEprmSectSize FAR* pEprmSectSize_p);


#ifdef _SUPPORT_PLC_CFG_SECT_
    // save PLC configuration to eeprom
    BYTE  EprmSavePlcCfg (BYTE FAR* pSrcData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p);

    // read PLC configuration from eeprom
    BYTE  EprmRecoverPlcCfg (BYTE FAR* pDstData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p);
#endif  // #ifdef _SUPPORT_PLC_CFG_SECT_


// set the data in the eeprom valid
BYTE  EprmSetDataValid (void);

// set the data in the eeprom invalid
BYTE  EprmSetDataInvalid (void);

// check for valid data in the eeprom
BYTE  EprmCheckDataValid (void);

// save data to the eeprom
BYTE  EprmSaveData (BYTE FAR* pSrcData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p);

// read data out of the eeprom
BYTE  EprmRecoverData (BYTE FAR* pDstData_p, DWORD dwDataOffset_p, DWORD dwDataSize_p);


#endif  // #ifndef _EEPRMDRV_H_

