/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  include file for rtc driver api function 

  -------------------------------------------------------------------------

                $RCSfile: RtcDrv.h,v $
                
                $Author: A.Suess $
                
                $Revision: 1.3 $  $Date: 2005/01/04 12:12:54 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/11/12 -ct:   start of the implementation

****************************************************************************/

#ifndef _RTCDRV_H_
#define _RTCDRV_H_

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define RTC_PORT              P9      // port9 of the microcontroller
#define RTC_DIR_PORT          DP9     // direction register of port 9

// SDA pin P9.4
#define SDA_LOW_MASK          0xFFEF  // mask in case of .AND. for low
#define SDA_HIGH_MASK         0x0010  // mask in case of .OR. for high

// SDA pin P9.5
#define SCL_LOW_MASK          0xFFDF  // mask in case of .AND. for low
#define SCL_HIGH_MASK         0x0020  // mask in case of .OR. for high


#define RTC_ADDR              0xA2    // I2C Address of the RTC

// register of the RTC
#define RTC_CNTRL1_REG        0x00
#define RTC_CNTRL2_REG        0x01
#define RTC_SECONDS_REG       0x02
#define RTC_MINUTES_REG       0x03
#define RTC_HOURS_REG         0x04
#define RTC_DAYS_REG          0x05
#define RTC_WEEKDAYS_REG      0x06
#define RTC_MONTH_REG         0x07
#define RTC_YEARS_REG         0x08
#define RTC_MINUTE_ALARM_REG  0x09
#define RTC_HOUR_ALARM_REG    0x0A
#define RTC_DAY_ALARM_REG     0x0B
#define RTC_WEEKDAY_ALARM_REG 0x0C
#define RTC_CLKOUT_REG        0x0D
#define RTC_TIMER_CNTRL_REG   0x0E
#define RTC_TIMER_REG         0x0F

#define RTC_SIZE_OF_REG        1
#define RTC_SIZE_OF_REAL_TIME  7
#define RTC_SIZE_OF_ALARM_TIME 4


//---------------------------------------------------------------------------
// standard data types
#ifndef  BCD8
#define  BCD8  unsigned char
#endif

#ifndef  BCD16
#define  BCD16 unsigned int
#endif


//---------------------------------------------------------------------------
//  defines of the day of week
#define  SUNDAY      0
#define  MONDAY      1
#define  TUESDAY     2
#define  WEDNESDAY   3
#define  THURSDAY    4
#define  FRIDAY      5
#define  SATURDAY    6

//---------------------------------------------------------------------------
//  errorcodes 
#define RTC_SUCCESSFUL       0x00
#define RTC_NOT_INITIALIZED  0xFD
#define RTC_WRITE_ERROR      0xFE
#define RTC_READ_ERROR       0xFF

//---------------------------------------------------------------------------
// types
//---------------------------------------------------------------------------

#ifndef _tWORD

#define _tWORD

    typedef union
    {
      WORD w;
      struct
        {
        BYTE low;
        BYTE high;
        } b;
    
    }tWORD;

#endif    

typedef struct 
{
    BYTE  bSecond;
    BYTE  bMinute;
    BYTE  bHour;
    BYTE  bDay;
    BYTE  bWeekday;
    BYTE  bMonth;
    WORD  wYears;
    BYTE  bTimeValid;

} tRealTime;


//---------------------------------------------------------------------------
// Alarm time

typedef struct 
{
    BYTE  bMinute;
    BYTE  bHour;
    BYTE  bDay;
    BYTE  bWeekday;

} tAlarmTime;


//---------------------------------------------------------------------------
// function prototypes
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Type fr Callback-Handler der Applikation
typedef void (PUBLIC ROM* tfpRtcCbHandler) (void);

//---------------------------------------------------------------------------
// function prototypes
//---------------------------------------------------------------------------

BYTE    PUBLIC RtcInit                (void);
BYTE    PUBLIC RtcSetAbsTime          (tRealTime FAR* pRealTime_p);
BYTE    PUBLIC RtcGetAbsTime          (tRealTime FAR* pRealTime_p);
void    PUBLIC RtcRelToAbsTime        (tRealTime FAR* pRealTime_p, DWORD dwRelTime_p);
DWORD   PUBLIC RtcAbsToRelTime        (tRealTime FAR* pRealTime_p);
BYTE    PUBLIC RtcGetDaysOfMonth      (BYTE bMonth, WORD wYear);

#endif  // #ifndef _RTCDRV_H_


